// GENERATED FILE
// ALL CHANGES MADE IN THIS FOLDER WILL BE LOST!

use tz::TimeZoneRef;

use super::{raw_tzdata, tzdata};
use crate::eq_ignore_ascii_case;

#[derive(Clone, Copy)]
#[repr(u16)]
pub(crate) enum Index {
    V0 = 1,
    V1 = 2,
    V2 = 3,
    V3 = 4,
    V4 = 5,
    V5 = 6,
    V6 = 7,
    V7 = 8,
    V8 = 9,
    V9 = 10,
    V10 = 11,
    V11 = 12,
    V12 = 13,
    V13 = 14,
    V14 = 15,
    V15 = 16,
    V16 = 17,
    V17 = 18,
    V18 = 19,
    V19 = 20,
    V20 = 21,
    V21 = 22,
    V22 = 23,
    V23 = 24,
    V24 = 25,
    V25 = 26,
    V26 = 27,
    V27 = 28,
    V28 = 29,
    V29 = 30,
    V30 = 31,
    V31 = 32,
    V32 = 33,
    V33 = 34,
    V34 = 35,
    V35 = 36,
    V36 = 37,
    V37 = 38,
    V38 = 39,
    V39 = 40,
    V40 = 41,
    V41 = 42,
    V42 = 43,
    V43 = 44,
    V44 = 45,
    V45 = 46,
    V46 = 47,
    V47 = 48,
    V48 = 49,
    V49 = 50,
    V50 = 51,
    V51 = 52,
    V52 = 53,
    V53 = 54,
    V54 = 55,
    V55 = 56,
    V56 = 57,
    V57 = 58,
    V58 = 59,
    V59 = 60,
    V60 = 61,
    V61 = 62,
    V62 = 63,
    V63 = 64,
    V64 = 65,
    V65 = 66,
    V66 = 67,
    V67 = 68,
    V68 = 69,
    V69 = 70,
    V70 = 71,
    V71 = 72,
    V72 = 73,
    V73 = 74,
    V74 = 75,
    V75 = 76,
    V76 = 77,
    V77 = 78,
    V78 = 79,
    V79 = 80,
    V80 = 81,
    V81 = 82,
    V82 = 83,
    V83 = 84,
    V84 = 85,
    V85 = 86,
    V86 = 87,
    V87 = 88,
    V88 = 89,
    V89 = 90,
    V90 = 91,
    V91 = 92,
    V92 = 93,
    V93 = 94,
    V94 = 95,
    V95 = 96,
    V96 = 97,
    V97 = 98,
    V98 = 99,
    V99 = 100,
    V100 = 101,
    V101 = 102,
    V102 = 103,
    V103 = 104,
    V104 = 105,
    V105 = 106,
    V106 = 107,
    V107 = 108,
    V108 = 109,
    V109 = 110,
    V110 = 111,
    V111 = 112,
    V112 = 113,
    V113 = 114,
    V114 = 115,
    V115 = 116,
    V116 = 117,
    V117 = 118,
    V118 = 119,
    V119 = 120,
    V120 = 121,
    V121 = 122,
    V122 = 123,
    V123 = 124,
    V124 = 125,
    V125 = 126,
    V126 = 127,
    V127 = 128,
    V128 = 129,
    V129 = 130,
    V130 = 131,
    V131 = 132,
    V132 = 133,
    V133 = 134,
    V134 = 135,
    V135 = 136,
    V136 = 137,
    V137 = 138,
    V138 = 139,
    V139 = 140,
    V140 = 141,
    V141 = 142,
    V142 = 143,
    V143 = 144,
    V144 = 145,
    V145 = 146,
    V146 = 147,
    V147 = 148,
    V148 = 149,
    V149 = 150,
    V150 = 151,
    V151 = 152,
    V152 = 153,
    V153 = 154,
    V154 = 155,
    V155 = 156,
    V156 = 157,
    V157 = 158,
    V158 = 159,
    V159 = 160,
    V160 = 161,
    V161 = 162,
    V162 = 163,
    V163 = 164,
    V164 = 165,
    V165 = 166,
    V166 = 167,
    V167 = 168,
    V168 = 169,
    V169 = 170,
    V170 = 171,
    V171 = 172,
    V172 = 173,
    V173 = 174,
    V174 = 175,
    V175 = 176,
    V176 = 177,
    V177 = 178,
    V178 = 179,
    V179 = 180,
    V180 = 181,
    V181 = 182,
    V182 = 183,
    V183 = 184,
    V184 = 185,
    V185 = 186,
    V186 = 187,
    V187 = 188,
    V188 = 189,
    V189 = 190,
    V190 = 191,
    V191 = 192,
    V192 = 193,
    V193 = 194,
    V194 = 195,
    V195 = 196,
    V196 = 197,
    V197 = 198,
    V198 = 199,
    V199 = 200,
    V200 = 201,
    V201 = 202,
    V202 = 203,
    V203 = 204,
    V204 = 205,
    V205 = 206,
    V206 = 207,
    V207 = 208,
    V208 = 209,
    V209 = 210,
    V210 = 211,
    V211 = 212,
    V212 = 213,
    V213 = 214,
    V214 = 215,
    V215 = 216,
    V216 = 217,
    V217 = 218,
    V218 = 219,
    V219 = 220,
    V220 = 221,
    V221 = 222,
    V222 = 223,
    V223 = 224,
    V224 = 225,
    V225 = 226,
    V226 = 227,
    V227 = 228,
    V228 = 229,
    V229 = 230,
    V230 = 231,
    V231 = 232,
    V232 = 233,
    V233 = 234,
    V234 = 235,
    V235 = 236,
    V236 = 237,
    V237 = 238,
    V238 = 239,
    V239 = 240,
    V240 = 241,
    V241 = 242,
    V242 = 243,
    V243 = 244,
    V244 = 245,
    V245 = 246,
    V246 = 247,
    V247 = 248,
    V248 = 249,
    V249 = 250,
    V250 = 251,
    V251 = 252,
    V252 = 253,
    V253 = 254,
    V254 = 255,
    V255 = 256,
    V256 = 257,
    V257 = 258,
    V258 = 259,
    V259 = 260,
    V260 = 261,
    V261 = 262,
    V262 = 263,
    V263 = 264,
    V264 = 265,
    V265 = 266,
    V266 = 267,
    V267 = 268,
    V268 = 269,
    V269 = 270,
    V270 = 271,
    V271 = 272,
    V272 = 273,
    V273 = 274,
    V274 = 275,
    V275 = 276,
    V276 = 277,
    V277 = 278,
    V278 = 279,
    V279 = 280,
    V280 = 281,
    V281 = 282,
    V282 = 283,
    V283 = 284,
    V284 = 285,
    V285 = 286,
    V286 = 287,
    V287 = 288,
    V288 = 289,
    V289 = 290,
    V290 = 291,
    V291 = 292,
    V292 = 293,
    V293 = 294,
    V294 = 295,
    V295 = 296,
    V296 = 297,
    V297 = 298,
    V298 = 299,
    V299 = 300,
    V300 = 301,
    V301 = 302,
    V302 = 303,
    V303 = 304,
    V304 = 305,
    V305 = 306,
    V306 = 307,
    V307 = 308,
    V308 = 309,
    V309 = 310,
    V310 = 311,
    V311 = 312,
    V312 = 313,
    V313 = 314,
    V314 = 315,
    V315 = 316,
    V316 = 317,
    V317 = 318,
    V318 = 319,
    V319 = 320,
    V320 = 321,
    V321 = 322,
    V322 = 323,
    V323 = 324,
    V324 = 325,
    V325 = 326,
    V326 = 327,
    V327 = 328,
    V328 = 329,
    V329 = 330,
    V330 = 331,
    V331 = 332,
    V332 = 333,
    V333 = 334,
    V334 = 335,
    V335 = 336,
    V336 = 337,
    V337 = 338,
    V338 = 339,
    V339 = 340,
    V340 = 341,
    V341 = 342,
    V342 = 343,
    V343 = 344,
    V344 = 345,
    V345 = 346,
    V346 = 347,
    V347 = 348,
    V348 = 349,
    V349 = 350,
    V350 = 351,
    V351 = 352,
    V352 = 353,
    V353 = 354,
    V354 = 355,
    V355 = 356,
    V356 = 357,
    V357 = 358,
    V358 = 359,
    V359 = 360,
    V360 = 361,
    V361 = 362,
    V362 = 363,
    V363 = 364,
    V364 = 365,
    V365 = 366,
    V366 = 367,
    V367 = 368,
    V368 = 369,
    V369 = 370,
    V370 = 371,
    V371 = 372,
    V372 = 373,
    V373 = 374,
    V374 = 375,
    V375 = 376,
    V376 = 377,
    V377 = 378,
    V378 = 379,
    V379 = 380,
    V380 = 381,
    V381 = 382,
    V382 = 383,
    V383 = 384,
    V384 = 385,
    V385 = 386,
    V386 = 387,
    V387 = 388,
    V388 = 389,
    V389 = 390,
    V390 = 391,
    V391 = 392,
    V392 = 393,
    V393 = 394,
    V394 = 395,
    V395 = 396,
    V396 = 397,
    V397 = 398,
    V398 = 399,
    V399 = 400,
    V400 = 401,
    V401 = 402,
    V402 = 403,
    V403 = 404,
    V404 = 405,
    V405 = 406,
    V406 = 407,
    V407 = 408,
    V408 = 409,
    V409 = 410,
    V410 = 411,
    V411 = 412,
    V412 = 413,
    V413 = 414,
    V414 = 415,
    V415 = 416,
    V416 = 417,
    V417 = 418,
    V418 = 419,
    V419 = 420,
    V420 = 421,
    V421 = 422,
    V422 = 423,
    V423 = 424,
    V424 = 425,
    V425 = 426,
    V426 = 427,
    V427 = 428,
    V428 = 429,
    V429 = 430,
    V430 = 431,
    V431 = 432,
    V432 = 433,
    V433 = 434,
    V434 = 435,
    V435 = 436,
    V436 = 437,
    V437 = 438,
    V438 = 439,
    V439 = 440,
    V440 = 441,
    V441 = 442,
    V442 = 443,
    V443 = 444,
    V444 = 445,
    V445 = 446,
    V446 = 447,
    V447 = 448,
    V448 = 449,
    V449 = 450,
    V450 = 451,
    V451 = 452,
    V452 = 453,
    V453 = 454,
    V454 = 455,
    V455 = 456,
    V456 = 457,
    V457 = 458,
    V458 = 459,
    V459 = 460,
    V460 = 461,
    V461 = 462,
    V462 = 463,
    V463 = 464,
    V464 = 465,
    V465 = 466,
    V466 = 467,
    V467 = 468,
    V468 = 469,
    V469 = 470,
    V470 = 471,
    V471 = 472,
    V472 = 473,
    V473 = 474,
    V474 = 475,
    V475 = 476,
    V476 = 477,
    V477 = 478,
    V478 = 479,
    V479 = 480,
    V480 = 481,
    V481 = 482,
    V482 = 483,
    V483 = 484,
    V484 = 485,
    V485 = 486,
    V486 = 487,
    V487 = 488,
    V488 = 489,
    V489 = 490,
    V490 = 491,
    V491 = 492,
    V492 = 493,
    V493 = 494,
    V494 = 495,
    V495 = 496,
    V496 = 497,
    V497 = 498,
    V498 = 499,
    V499 = 500,
    V500 = 501,
    V501 = 502,
    V502 = 503,
    V503 = 504,
    V504 = 505,
    V505 = 506,
    V506 = 507,
    V507 = 508,
    V508 = 509,
    V509 = 510,
    V510 = 511,
    V511 = 512,
    V512 = 513,
    V513 = 514,
    V514 = 515,
    V515 = 516,
    V516 = 517,
    V517 = 518,
    V518 = 519,
    V519 = 520,
    V520 = 521,
    V521 = 522,
    V522 = 523,
    V523 = 524,
    V524 = 525,
    V525 = 526,
    V526 = 527,
    V527 = 528,
    V528 = 529,
    V529 = 530,
    V530 = 531,
    V531 = 532,
    V532 = 533,
    V533 = 534,
    V534 = 535,
    V535 = 536,
    V536 = 537,
    V537 = 538,
    V538 = 539,
    V539 = 540,
    V540 = 541,
    V541 = 542,
    V542 = 543,
    V543 = 544,
    V544 = 545,
    V545 = 546,
    V546 = 547,
    V547 = 548,
    V548 = 549,
    V549 = 550,
    V550 = 551,
    V551 = 552,
    V552 = 553,
    V553 = 554,
    V554 = 555,
    V555 = 556,
    V556 = 557,
    V557 = 558,
    V558 = 559,
    V559 = 560,
    V560 = 561,
    V561 = 562,
    V562 = 563,
    V563 = 564,
    V564 = 565,
    V565 = 566,
    V566 = 567,
    V567 = 568,
    V568 = 569,
    V569 = 570,
    V570 = 571,
    V571 = 572,
    V572 = 573,
    V573 = 574,
    V574 = 575,
    V575 = 576,
    V576 = 577,
    V577 = 578,
    V578 = 579,
    V579 = 580,
    V580 = 581,
    V581 = 582,
    V582 = 583,
    V583 = 584,
    V584 = 585,
    V585 = 586,
    V586 = 587,
    V587 = 588,
    V588 = 589,
    V589 = 590,
    V590 = 591,
    V591 = 592,
    V592 = 593,
    V593 = 594,
    V594 = 595,
    V595 = 596,
    V596 = 597,
    V597 = 598,
}

const WORDLIST: [Option<Index>; 2522] = [
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V0),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V1),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V2),
    Some(Index::V3),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V4),
    None,
    None,
    None,
    Some(Index::V5),
    None,
    Some(Index::V6),
    Some(Index::V7),
    None,
    Some(Index::V8),
    Some(Index::V9),
    Some(Index::V10),
    Some(Index::V11),
    Some(Index::V12),
    Some(Index::V13),
    None,
    None,
    None,
    Some(Index::V14),
    None,
    None,
    Some(Index::V15),
    None,
    Some(Index::V16),
    None,
    None,
    None,
    Some(Index::V17),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V18),
    None,
    Some(Index::V19),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V20),
    None,
    None,
    Some(Index::V21),
    None,
    Some(Index::V22),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V23),
    Some(Index::V24),
    None,
    None,
    None,
    Some(Index::V25),
    None,
    None,
    Some(Index::V26),
    None,
    None,
    Some(Index::V27),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V28),
    None,
    None,
    Some(Index::V29),
    Some(Index::V30),
    Some(Index::V31),
    None,
    None,
    None,
    Some(Index::V32),
    None,
    None,
    None,
    None,
    None,
    Some(Index::V33),
    None,
    Some(Index::V34),
    None,
    None,
    Some(Index::V35),
    Some(Index::V36),
    Some(Index::V37),
    Some(Index::V38),
    None,
    None,
    None,
    None,
    Some(Index::V39),
    Some(Index::V40),
    None,
    Some(Index::V41),
    None,
    Some(Index::V42),
    Some(Index::V43),
    Some(Index::V44),
    None,
    Some(Index::V45),
    None,
    Some(Index::V46),
    None,
    Some(Index::V47),
    None,
    None,
    None,
    Some(Index::V48),
    None,
    None,
    Some(Index::V49),
    None,
    None,
    Some(Index::V50),
    Some(Index::V51),
    Some(Index::V52),
    None,
    None,
    None,
    Some(Index::V53),
    Some(Index::V54),
    Some(Index::V55),
    Some(Index::V56),
    Some(Index::V57),
    Some(Index::V58),
    None,
    Some(Index::V59),
    Some(Index::V60),
    Some(Index::V61),
    Some(Index::V62),
    Some(Index::V63),
    None,
    None,
    Some(Index::V64),
    None,
    Some(Index::V65),
    None,
    Some(Index::V66),
    None,
    None,
    None,
    Some(Index::V67),
    Some(Index::V68),
    None,
    Some(Index::V69),
    None,
    Some(Index::V70),
    None,
    None,
    None,
    None,
    None,
    Some(Index::V71),
    Some(Index::V72),
    Some(Index::V73),
    Some(Index::V74),
    None,
    Some(Index::V75),
    None,
    Some(Index::V76),
    Some(Index::V77),
    None,
    Some(Index::V78),
    None,
    None,
    None,
    Some(Index::V79),
    Some(Index::V80),
    Some(Index::V81),
    None,
    Some(Index::V82),
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V83),
    None,
    None,
    None,
    None,
    None,
    Some(Index::V84),
    None,
    Some(Index::V85),
    None,
    Some(Index::V86),
    Some(Index::V87),
    Some(Index::V88),
    None,
    Some(Index::V89),
    Some(Index::V90),
    None,
    Some(Index::V91),
    None,
    None,
    Some(Index::V92),
    None,
    Some(Index::V93),
    Some(Index::V94),
    None,
    None,
    None,
    Some(Index::V95),
    Some(Index::V96),
    None,
    None,
    Some(Index::V97),
    Some(Index::V98),
    Some(Index::V99),
    None,
    Some(Index::V100),
    None,
    None,
    Some(Index::V101),
    None,
    Some(Index::V102),
    None,
    None,
    Some(Index::V103),
    None,
    None,
    None,
    None,
    None,
    Some(Index::V104),
    Some(Index::V105),
    None,
    None,
    None,
    Some(Index::V106),
    None,
    Some(Index::V107),
    None,
    Some(Index::V108),
    Some(Index::V109),
    Some(Index::V110),
    Some(Index::V111),
    Some(Index::V112),
    None,
    Some(Index::V113),
    Some(Index::V114),
    Some(Index::V115),
    Some(Index::V116),
    Some(Index::V117),
    None,
    Some(Index::V118),
    Some(Index::V119),
    Some(Index::V120),
    Some(Index::V121),
    None,
    Some(Index::V122),
    Some(Index::V123),
    None,
    Some(Index::V124),
    None,
    Some(Index::V125),
    None,
    Some(Index::V126),
    None,
    Some(Index::V127),
    Some(Index::V128),
    None,
    Some(Index::V129),
    Some(Index::V130),
    Some(Index::V131),
    None,
    Some(Index::V132),
    None,
    None,
    None,
    None,
    Some(Index::V133),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V134),
    None,
    Some(Index::V135),
    None,
    Some(Index::V136),
    None,
    Some(Index::V137),
    None,
    None,
    Some(Index::V138),
    None,
    Some(Index::V139),
    Some(Index::V140),
    Some(Index::V141),
    None,
    Some(Index::V142),
    None,
    Some(Index::V143),
    None,
    Some(Index::V144),
    None,
    None,
    Some(Index::V145),
    Some(Index::V146),
    Some(Index::V147),
    None,
    Some(Index::V148),
    None,
    Some(Index::V149),
    None,
    None,
    None,
    None,
    Some(Index::V150),
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V151),
    None,
    Some(Index::V152),
    None,
    Some(Index::V153),
    Some(Index::V154),
    None,
    Some(Index::V155),
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V156),
    None,
    None,
    None,
    Some(Index::V157),
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V158),
    None,
    Some(Index::V159),
    None,
    None,
    Some(Index::V160),
    None,
    None,
    None,
    Some(Index::V161),
    Some(Index::V162),
    Some(Index::V163),
    Some(Index::V164),
    Some(Index::V165),
    Some(Index::V166),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V167),
    None,
    Some(Index::V168),
    Some(Index::V169),
    Some(Index::V170),
    None,
    None,
    None,
    None,
    None,
    Some(Index::V171),
    None,
    Some(Index::V172),
    None,
    None,
    None,
    None,
    Some(Index::V173),
    None,
    None,
    None,
    Some(Index::V174),
    Some(Index::V175),
    None,
    None,
    None,
    Some(Index::V176),
    Some(Index::V177),
    None,
    Some(Index::V178),
    Some(Index::V179),
    None,
    Some(Index::V180),
    Some(Index::V181),
    Some(Index::V182),
    Some(Index::V183),
    Some(Index::V184),
    None,
    None,
    None,
    Some(Index::V185),
    None,
    None,
    None,
    Some(Index::V186),
    None,
    None,
    None,
    Some(Index::V187),
    None,
    Some(Index::V188),
    None,
    Some(Index::V189),
    None,
    Some(Index::V190),
    Some(Index::V191),
    Some(Index::V192),
    None,
    Some(Index::V193),
    None,
    None,
    None,
    None,
    None,
    Some(Index::V194),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V195),
    None,
    Some(Index::V196),
    None,
    None,
    None,
    None,
    Some(Index::V197),
    None,
    None,
    Some(Index::V198),
    None,
    Some(Index::V199),
    Some(Index::V200),
    None,
    None,
    Some(Index::V201),
    None,
    None,
    None,
    None,
    None,
    Some(Index::V202),
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V203),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V204),
    None,
    None,
    None,
    Some(Index::V205),
    Some(Index::V206),
    None,
    None,
    Some(Index::V207),
    None,
    None,
    None,
    Some(Index::V208),
    None,
    None,
    Some(Index::V209),
    None,
    None,
    Some(Index::V210),
    None,
    None,
    None,
    None,
    Some(Index::V211),
    None,
    None,
    Some(Index::V212),
    None,
    Some(Index::V213),
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V214),
    None,
    None,
    None,
    Some(Index::V215),
    Some(Index::V216),
    Some(Index::V217),
    Some(Index::V218),
    None,
    Some(Index::V219),
    None,
    None,
    None,
    None,
    None,
    Some(Index::V220),
    None,
    None,
    Some(Index::V221),
    None,
    None,
    None,
    Some(Index::V222),
    None,
    None,
    None,
    Some(Index::V223),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V224),
    Some(Index::V225),
    Some(Index::V226),
    Some(Index::V227),
    Some(Index::V228),
    Some(Index::V229),
    None,
    Some(Index::V230),
    Some(Index::V231),
    Some(Index::V232),
    None,
    Some(Index::V233),
    Some(Index::V234),
    None,
    Some(Index::V235),
    Some(Index::V236),
    None,
    Some(Index::V237),
    Some(Index::V238),
    Some(Index::V239),
    None,
    Some(Index::V240),
    Some(Index::V241),
    Some(Index::V242),
    Some(Index::V243),
    Some(Index::V244),
    Some(Index::V245),
    None,
    None,
    Some(Index::V246),
    Some(Index::V247),
    Some(Index::V248),
    None,
    Some(Index::V249),
    Some(Index::V250),
    Some(Index::V251),
    Some(Index::V252),
    Some(Index::V253),
    Some(Index::V254),
    Some(Index::V255),
    None,
    Some(Index::V256),
    None,
    Some(Index::V257),
    None,
    Some(Index::V258),
    Some(Index::V259),
    Some(Index::V260),
    Some(Index::V261),
    Some(Index::V262),
    None,
    Some(Index::V263),
    None,
    Some(Index::V264),
    None,
    None,
    Some(Index::V265),
    None,
    None,
    Some(Index::V266),
    Some(Index::V267),
    Some(Index::V268),
    Some(Index::V269),
    Some(Index::V270),
    None,
    Some(Index::V271),
    None,
    Some(Index::V272),
    Some(Index::V273),
    Some(Index::V274),
    Some(Index::V275),
    None,
    Some(Index::V276),
    None,
    Some(Index::V277),
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V278),
    None,
    None,
    Some(Index::V279),
    Some(Index::V280),
    None,
    None,
    None,
    None,
    Some(Index::V281),
    Some(Index::V282),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V283),
    None,
    Some(Index::V284),
    None,
    None,
    None,
    Some(Index::V285),
    Some(Index::V286),
    Some(Index::V287),
    None,
    None,
    None,
    Some(Index::V288),
    None,
    Some(Index::V289),
    None,
    None,
    None,
    None,
    Some(Index::V290),
    None,
    Some(Index::V291),
    None,
    Some(Index::V292),
    Some(Index::V293),
    Some(Index::V294),
    None,
    None,
    None,
    None,
    Some(Index::V295),
    None,
    Some(Index::V296),
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V297),
    None,
    None,
    None,
    None,
    Some(Index::V298),
    None,
    None,
    None,
    Some(Index::V299),
    Some(Index::V300),
    Some(Index::V301),
    Some(Index::V302),
    None,
    Some(Index::V303),
    Some(Index::V304),
    Some(Index::V305),
    Some(Index::V306),
    Some(Index::V307),
    None,
    Some(Index::V308),
    Some(Index::V309),
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V310),
    None,
    Some(Index::V311),
    None,
    None,
    None,
    None,
    Some(Index::V312),
    Some(Index::V313),
    None,
    None,
    Some(Index::V314),
    Some(Index::V315),
    None,
    Some(Index::V316),
    None,
    None,
    Some(Index::V317),
    Some(Index::V318),
    None,
    Some(Index::V319),
    Some(Index::V320),
    None,
    None,
    None,
    None,
    None,
    Some(Index::V321),
    None,
    Some(Index::V322),
    Some(Index::V323),
    Some(Index::V324),
    Some(Index::V325),
    Some(Index::V326),
    None,
    None,
    Some(Index::V327),
    None,
    Some(Index::V328),
    None,
    Some(Index::V329),
    None,
    Some(Index::V330),
    None,
    None,
    Some(Index::V331),
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V332),
    Some(Index::V333),
    None,
    None,
    Some(Index::V334),
    None,
    Some(Index::V335),
    Some(Index::V336),
    None,
    Some(Index::V337),
    None,
    None,
    Some(Index::V338),
    None,
    None,
    None,
    Some(Index::V339),
    Some(Index::V340),
    Some(Index::V341),
    None,
    Some(Index::V342),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V343),
    None,
    None,
    Some(Index::V344),
    None,
    None,
    None,
    None,
    None,
    Some(Index::V345),
    Some(Index::V346),
    None,
    Some(Index::V347),
    None,
    None,
    None,
    None,
    Some(Index::V348),
    None,
    Some(Index::V349),
    Some(Index::V350),
    None,
    Some(Index::V351),
    Some(Index::V352),
    None,
    None,
    Some(Index::V353),
    None,
    Some(Index::V354),
    Some(Index::V355),
    None,
    None,
    None,
    Some(Index::V356),
    None,
    Some(Index::V357),
    Some(Index::V358),
    None,
    None,
    Some(Index::V359),
    None,
    Some(Index::V360),
    None,
    Some(Index::V361),
    None,
    None,
    Some(Index::V362),
    None,
    Some(Index::V363),
    Some(Index::V364),
    Some(Index::V365),
    None,
    None,
    Some(Index::V366),
    Some(Index::V367),
    None,
    None,
    Some(Index::V368),
    None,
    None,
    Some(Index::V369),
    None,
    None,
    Some(Index::V370),
    Some(Index::V371),
    None,
    None,
    None,
    Some(Index::V372),
    Some(Index::V373),
    None,
    None,
    None,
    None,
    None,
    Some(Index::V374),
    None,
    None,
    None,
    Some(Index::V375),
    Some(Index::V376),
    None,
    Some(Index::V377),
    None,
    Some(Index::V378),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V379),
    None,
    Some(Index::V380),
    Some(Index::V381),
    None,
    None,
    None,
    None,
    None,
    Some(Index::V382),
    None,
    None,
    Some(Index::V383),
    None,
    None,
    None,
    Some(Index::V384),
    Some(Index::V385),
    None,
    Some(Index::V386),
    None,
    Some(Index::V387),
    Some(Index::V388),
    None,
    Some(Index::V389),
    Some(Index::V390),
    Some(Index::V391),
    None,
    None,
    Some(Index::V392),
    None,
    None,
    None,
    Some(Index::V393),
    None,
    Some(Index::V394),
    None,
    Some(Index::V395),
    Some(Index::V396),
    None,
    Some(Index::V397),
    None,
    Some(Index::V398),
    Some(Index::V399),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V400),
    None,
    Some(Index::V401),
    Some(Index::V402),
    Some(Index::V403),
    None,
    Some(Index::V404),
    None,
    Some(Index::V405),
    None,
    None,
    None,
    Some(Index::V406),
    None,
    None,
    None,
    Some(Index::V407),
    Some(Index::V408),
    Some(Index::V409),
    None,
    Some(Index::V410),
    Some(Index::V411),
    None,
    Some(Index::V412),
    Some(Index::V413),
    None,
    None,
    None,
    Some(Index::V414),
    None,
    None,
    None,
    Some(Index::V415),
    None,
    Some(Index::V416),
    Some(Index::V417),
    Some(Index::V418),
    Some(Index::V419),
    Some(Index::V420),
    None,
    None,
    None,
    None,
    Some(Index::V421),
    None,
    None,
    None,
    None,
    None,
    Some(Index::V422),
    Some(Index::V423),
    None,
    None,
    None,
    None,
    None,
    Some(Index::V424),
    None,
    Some(Index::V425),
    None,
    Some(Index::V426),
    None,
    None,
    None,
    Some(Index::V427),
    Some(Index::V428),
    Some(Index::V429),
    None,
    Some(Index::V430),
    None,
    None,
    None,
    Some(Index::V431),
    None,
    None,
    Some(Index::V432),
    None,
    None,
    Some(Index::V433),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V434),
    None,
    Some(Index::V435),
    None,
    None,
    None,
    Some(Index::V436),
    None,
    None,
    None,
    None,
    Some(Index::V437),
    None,
    Some(Index::V438),
    Some(Index::V439),
    None,
    None,
    Some(Index::V440),
    None,
    None,
    Some(Index::V441),
    None,
    None,
    None,
    None,
    None,
    Some(Index::V442),
    Some(Index::V443),
    None,
    None,
    None,
    Some(Index::V444),
    None,
    None,
    None,
    Some(Index::V445),
    None,
    None,
    None,
    None,
    None,
    Some(Index::V446),
    None,
    None,
    None,
    None,
    Some(Index::V447),
    None,
    Some(Index::V448),
    Some(Index::V449),
    None,
    None,
    None,
    None,
    None,
    Some(Index::V450),
    None,
    None,
    None,
    Some(Index::V451),
    None,
    None,
    None,
    None,
    Some(Index::V452),
    None,
    Some(Index::V453),
    None,
    None,
    Some(Index::V454),
    Some(Index::V455),
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V456),
    None,
    None,
    None,
    Some(Index::V457),
    Some(Index::V458),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V459),
    None,
    None,
    Some(Index::V460),
    None,
    Some(Index::V461),
    None,
    None,
    Some(Index::V462),
    None,
    None,
    None,
    Some(Index::V463),
    None,
    None,
    None,
    None,
    Some(Index::V464),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V465),
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V466),
    Some(Index::V467),
    None,
    Some(Index::V468),
    None,
    Some(Index::V469),
    Some(Index::V470),
    Some(Index::V471),
    Some(Index::V472),
    None,
    None,
    None,
    Some(Index::V473),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V474),
    None,
    Some(Index::V475),
    None,
    Some(Index::V476),
    None,
    None,
    None,
    Some(Index::V477),
    Some(Index::V478),
    None,
    Some(Index::V479),
    None,
    None,
    None,
    Some(Index::V480),
    None,
    None,
    None,
    Some(Index::V481),
    None,
    None,
    None,
    None,
    Some(Index::V482),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V483),
    Some(Index::V484),
    None,
    None,
    Some(Index::V485),
    None,
    None,
    None,
    None,
    None,
    Some(Index::V486),
    None,
    None,
    None,
    Some(Index::V487),
    None,
    Some(Index::V488),
    Some(Index::V489),
    None,
    None,
    None,
    None,
    Some(Index::V490),
    None,
    None,
    Some(Index::V491),
    Some(Index::V492),
    None,
    None,
    None,
    None,
    None,
    Some(Index::V493),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V494),
    None,
    None,
    None,
    None,
    Some(Index::V495),
    Some(Index::V496),
    Some(Index::V497),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V498),
    None,
    None,
    Some(Index::V499),
    None,
    None,
    Some(Index::V500),
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V501),
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V502),
    Some(Index::V503),
    Some(Index::V504),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V505),
    None,
    Some(Index::V506),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V507),
    None,
    Some(Index::V508),
    None,
    Some(Index::V509),
    None,
    None,
    None,
    Some(Index::V510),
    None,
    Some(Index::V511),
    None,
    None,
    None,
    Some(Index::V512),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V513),
    None,
    None,
    None,
    Some(Index::V514),
    Some(Index::V515),
    Some(Index::V516),
    None,
    None,
    None,
    None,
    None,
    Some(Index::V517),
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V518),
    None,
    None,
    None,
    None,
    None,
    Some(Index::V519),
    None,
    None,
    Some(Index::V520),
    None,
    None,
    Some(Index::V521),
    Some(Index::V522),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V523),
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V524),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V525),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V526),
    None,
    None,
    Some(Index::V527),
    None,
    None,
    Some(Index::V528),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V529),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V530),
    None,
    None,
    None,
    Some(Index::V531),
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V532),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V533),
    None,
    None,
    None,
    None,
    None,
    Some(Index::V534),
    None,
    None,
    Some(Index::V535),
    None,
    None,
    None,
    None,
    Some(Index::V536),
    None,
    None,
    None,
    None,
    Some(Index::V537),
    None,
    None,
    Some(Index::V538),
    None,
    None,
    Some(Index::V539),
    None,
    None,
    Some(Index::V540),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V541),
    None,
    None,
    None,
    None,
    Some(Index::V542),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V543),
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V544),
    None,
    None,
    Some(Index::V545),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V546),
    None,
    None,
    None,
    Some(Index::V547),
    None,
    None,
    None,
    Some(Index::V548),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V549),
    None,
    Some(Index::V550),
    Some(Index::V551),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V552),
    Some(Index::V553),
    Some(Index::V554),
    None,
    None,
    None,
    None,
    Some(Index::V555),
    None,
    Some(Index::V556),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V557),
    None,
    None,
    None,
    Some(Index::V558),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V559),
    Some(Index::V560),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V561),
    None,
    None,
    None,
    Some(Index::V562),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V563),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V564),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V565),
    Some(Index::V566),
    None,
    Some(Index::V567),
    None,
    None,
    None,
    None,
    Some(Index::V568),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V569),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V570),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V571),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V572),
    None,
    None,
    None,
    Some(Index::V573),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V574),
    None,
    None,
    None,
    Some(Index::V575),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V576),
    Some(Index::V577),
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V578),
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V579),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V580),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V581),
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V582),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V583),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V584),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V585),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V586),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V587),
    None,
    None,
    Some(Index::V588),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V589),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V590),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V591),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V592),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V593),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V594),
    None,
    None,
    None,
    Some(Index::V595),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V596),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    Some(Index::V597),
];

const NAMES: [&[u8]; 599] = [
    b"",
    b"EST",
    b"EST5EDT",
    b"Asia/Manila",
    b"Asia/Macao",
    b"Asia/Amman",
    b"Indian/Mahe",
    b"GMT",
    b"Asia/Nicosia",
    b"America/Tortola",
    b"Indian/Mayotte",
    b"GMT0",
    b"GMT-0",
    b"GMT+0",
    b"America/Antigua",
    b"America/Toronto",
    b"America/Noronha",
    b"America/Ojinaga",
    b"America/Detroit",
    b"America/North_Dakota/Center",
    b"Indian/Comoro",
    b"America/Rainy_River",
    b"America/Blanc-Sablon",
    b"Indian/Antananarivo",
    b"Asia/Anadyr",
    b"America/Indiana/Marengo",
    b"America/Guatemala",
    b"America/Araguaina",
    b"America/Grenada",
    b"Iran",
    b"America/Argentina/Tucuman",
    b"UCT",
    b"America/Regina",
    b"America/Guadeloupe",
    b"Africa/Porto-Novo",
    b"Africa/Ceuta",
    b"Africa/Dakar",
    b"Africa/Monrovia",
    b"America/Bogota",
    b"Africa/Harare",
    b"Africa/Nairobi",
    b"Asia/Ashgabat",
    b"Asia/Qatar",
    b"Africa/Cairo",
    b"Asia/Macau",
    b"America/Godthab",
    b"America/Indiana/Knox",
    b"America/Indiana/Petersburg",
    b"Africa/Malabo",
    b"Asia/Tehran",
    b"Asia/Muscat",
    b"America/Santiago",
    b"America/Ensenada",
    b"CST6CDT",
    b"Africa/Casablanca",
    b"Asia/Harbin",
    b"America/Rosario",
    b"Asia/Aqtobe",
    b"Asia/Dubai",
    b"America/Resolute",
    b"America/Santo_Domingo",
    b"Indian/Reunion",
    b"America/Asuncion",
    b"US/East-Indiana",
    b"Asia/Dacca",
    b"Indian/Mauritius",
    b"America/Eirunepe",
    b"Africa/Mbabane",
    b"Indian/Maldives",
    b"Asia/Hebron",
    b"NZ-CHAT",
    b"Africa/Freetown",
    b"America/Scoresbysund",
    b"America/Aruba",
    b"America/Boise",
    b"America/Catamarca",
    b"Asia/Rangoon",
    b"Asia/Bahrain",
    b"America/Buenos_Aires",
    b"MST",
    b"America/Cuiaba",
    b"America/Indiana/Vincennes",
    b"America/Curacao",
    b"America/Indianapolis",
    b"America/Recife",
    b"America/Cordoba",
    b"America/Indiana/Indianapolis",
    b"America/Barbados",
    b"America/Coral_Harbour",
    b"GB",
    b"MST7MDT",
    b"America/Anguilla",
    b"America/Argentina/Salta",
    b"America/Tegucigalpa",
    b"Eire",
    b"America/Argentina/San_Juan",
    b"Asia/Hovd",
    b"Canada/Mountain",
    b"America/Argentina/Ushuaia",
    b"Africa/Maputo",
    b"America/Argentina/Rio_Gallegos",
    b"America/Indiana/Winamac",
    b"Asia/Dili",
    b"America/Whitehorse",
    b"Africa/Maseru",
    b"Asia/Chita",
    b"PST8PDT",
    b"Japan",
    b"Africa/Mogadishu",
    b"America/Argentina/Buenos_Aires",
    b"US/Samoa",
    b"Africa/Asmara",
    b"America/Montserrat",
    b"Africa/Asmera",
    b"America/Managua",
    b"America/Nome",
    b"America/Argentina/Cordoba",
    b"America/Montevideo",
    b"America/Argentina/Catamarca",
    b"America/Edmonton",
    b"America/Menominee",
    b"America/Martinique",
    b"America/Argentina/ComodRivadavia",
    b"Asia/Aqtau",
    b"Jamaica",
    b"America/Marigot",
    b"America/Merida",
    b"America/Mendoza",
    b"Indian/Cocos",
    b"Africa/Addis_Ababa",
    b"America/North_Dakota/New_Salem",
    b"Asia/Colombo",
    b"Navajo",
    b"America/Dominica",
    b"US/Indiana-Starke",
    b"America/Winnipeg",
    b"Africa/Accra",
    b"America/Panama",
    b"America/Costa_Rica",
    b"Asia/Saigon",
    b"America/Creston",
    b"Asia/Famagusta",
    b"America/Porto_Acre",
    b"America/Caracas",
    b"America/Paramaribo",
    b"America/Nipigon",
    b"America/Port_of_Spain",
    b"America/Porto_Velho",
    b"GB-Eire",
    b"Iceland",
    b"America/Puerto_Rico",
    b"America/Port-au-Prince",
    b"America/Iqaluit",
    b"Africa/Lome",
    b"America/Argentina/Mendoza",
    b"Asia/Singapore",
    b"America/Argentina/San_Luis",
    b"Asia/Kuwait",
    b"America/Cancun",
    b"Africa/Ndjamena",
    b"America/Chicago",
    b"Africa/Libreville",
    b"America/Dawson",
    b"America/Phoenix",
    b"Asia/Chongqing",
    b"NZ",
    b"America/Belize",
    b"America/Denver",
    b"America/Matamoros",
    b"America/Santarem",
    b"America/Manaus",
    b"UTC",
    b"Asia/Ulaanbaatar",
    b"Asia/Ulan_Bator",
    b"America/Nassau",
    b"America/Miquelon",
    b"Asia/Qostanay",
    b"Africa/Djibouti",
    b"Asia/Calcutta",
    b"America/Tijuana",
    b"Asia/Beirut",
    b"ROC",
    b"America/Argentina/La_Rioja",
    b"Asia/Damascus",
    b"Africa/Douala",
    b"America/Juneau",
    b"Canada/Atlantic",
    b"America/Virgin",
    b"Asia/Qyzylorda",
    b"EET",
    b"America/Maceio",
    b"America/Moncton",
    b"Asia/Makassar",
    b"America/Punta_Arenas",
    b"W-SU",
    b"Africa/Abidjan",
    b"US/Alaska",
    b"Cuba",
    b"Asia/Ust-Nera",
    b"America/Thule",
    b"Asia/Yangon",
    b"America/Santa_Isabel",
    b"Asia/Kamchatka",
    b"Africa/Bangui",
    b"America/Louisville",
    b"Asia/Aden",
    b"America/Guyana",
    b"Africa/Luanda",
    b"America/Rio_Branco",
    b"Asia/Brunei",
    b"America/Fortaleza",
    b"America/Fort_Wayne",
    b"America/Pangnirtung",
    b"America/Boa_Vista",
    b"America/Fort_Nelson",
    b"America/Jamaica",
    b"Asia/Yerevan",
    b"PRC",
    b"America/Vancouver",
    b"Israel",
    b"Africa/Bissau",
    b"Indian/Kerguelen",
    b"Africa/Tripoli",
    b"Africa/Timbuktu",
    b"Etc/GMT",
    b"Poland",
    b"Etc/GMT0",
    b"Singapore",
    b"Etc/GMT-0",
    b"Africa/Sao_Tome",
    b"Canada/Central",
    b"Etc/UCT",
    b"Etc/GMT-10",
    b"Etc/GMT+0",
    b"Etc/GMT-1",
    b"Etc/GMT-11",
    b"Etc/GMT+10",
    b"Etc/GMT-8",
    b"Etc/GMT+1",
    b"Etc/GMT+11",
    b"America/Lima",
    b"Etc/GMT+8",
    b"America/Metlakatla",
    b"America/Montreal",
    b"Etc/GMT-12",
    b"Pacific/Efate",
    b"Africa/Algiers",
    b"Africa/Blantyre",
    b"Etc/GMT+12",
    b"Pacific/Tarawa",
    b"Etc/GMT-7",
    b"Pacific/Apia",
    b"Etc/GMT-2",
    b"Africa/Lagos",
    b"Etc/GMT-14",
    b"Etc/GMT-13",
    b"Etc/GMT+7",
    b"Etc/GMT+2",
    b"CET",
    b"Etc/GMT-6",
    b"US/Eastern",
    b"Etc/GMT-5",
    b"America/Cayenne",
    b"Pacific/Rarotonga",
    b"Etc/GMT+6",
    b"Etc/GMT+5",
    b"America/Belem",
    b"Etc/GMT-9",
    b"America/Sao_Paulo",
    b"Etc/GMT-4",
    b"Australia/Tasmania",
    b"Etc/GMT-3",
    b"Etc/GMT+9",
    b"Etc/GMT+4",
    b"Asia/Choibalsan",
    b"Etc/GMT+3",
    b"Africa/Tunis",
    b"America/Goose_Bay",
    b"America/Danmarkshavn",
    b"America/El_Salvador",
    b"Australia/Darwin",
    b"Australia/Adelaide",
    b"America/Swift_Current",
    b"WET",
    b"HST",
    b"US/Pacific",
    b"Australia/Brisbane",
    b"US/Arizona",
    b"Asia/Pontianak",
    b"America/Glace_Bay",
    b"Chile/EasterIsland",
    b"Asia/Samarkand",
    b"MET",
    b"Asia/Dhaka",
    b"America/Monterrey",
    b"America/Mazatlan",
    b"America/Atka",
    b"Asia/Taipei",
    b"Asia/Gaza",
    b"Asia/Vientiane",
    b"America/Atikokan",
    b"Africa/Kampala",
    b"Asia/Atyrau",
    b"Asia/Magadan",
    b"Canada/Eastern",
    b"America/Rankin_Inlet",
    b"America/Kentucky/Louisville",
    b"America/Kentucky/Monticello",
    b"Asia/Seoul",
    b"US/Michigan",
    b"Pacific/Easter",
    b"US/Central",
    b"Pacific/Niue",
    b"Africa/Ouagadougou",
    b"Asia/Baku",
    b"America/St_Thomas",
    b"Atlantic/Faeroe",
    b"Asia/Dushanbe",
    b"Atlantic/Jan_Mayen",
    b"Atlantic/Faroe",
    b"Atlantic/Bermuda",
    b"Asia/Shanghai",
    b"Atlantic/Madeira",
    b"America/Campo_Grande",
    b"US/Aleutian",
    b"Asia/Ashkhabad",
    b"America/North_Dakota/Beulah",
    b"Africa/Kigali",
    b"Australia/Eucla",
    b"Canada/Newfoundland",
    b"America/Knox_IN",
    b"America/Grand_Turk",
    b"Asia/Pyongyang",
    b"America/Bahia",
    b"America/Ciudad_Juarez",
    b"Africa/Niamey",
    b"Africa/Nouakchott",
    b"America/Argentina/Jujuy",
    b"Canada/Yukon",
    b"Australia/ACT",
    b"Africa/Bujumbura",
    b"Australia/Queensland",
    b"Pacific/Tongatapu",
    b"Australia/Canberra",
    b"Asia/Yekaterinburg",
    b"America/Indiana/Tell_City",
    b"Asia/Hong_Kong",
    b"America/Sitka",
    b"Europe/Malta",
    b"America/Indiana/Vevay",
    b"Europe/Samara",
    b"America/Guayaquil",
    b"Asia/Khandyga",
    b"Europe/Sofia",
    b"Brazil/DeNoronha",
    b"Africa/Gaborone",
    b"Africa/Dar_es_Salaam",
    b"Australia/West",
    b"Europe/Mariehamn",
    b"Asia/Tashkent",
    b"US/Mountain",
    b"Europe/Zaporozhye",
    b"Asia/Katmandu",
    b"Europe/Copenhagen",
    b"Asia/Tokyo",
    b"Asia/Almaty",
    b"Asia/Tbilisi",
    b"America/Cayman",
    b"Europe/Madrid",
    b"America/Lower_Princes",
    b"Etc/UTC",
    b"Europe/Andorra",
    b"Brazil/West",
    b"Europe/Zagreb",
    b"Pacific/Nauru",
    b"Asia/Istanbul",
    b"Arctic/Longyearbyen",
    b"Africa/Lubumbashi",
    b"Pacific/Ponape",
    b"Australia/Melbourne",
    b"Atlantic/South_Georgia",
    b"Canada/Pacific",
    b"Europe/Podgorica",
    b"Asia/Thimbu",
    b"Africa/Juba",
    b"America/Anchorage",
    b"Africa/El_Aaiun",
    b"Antarctica/Rothera",
    b"Canada/Saskatchewan",
    b"Asia/Tomsk",
    b"America/Bahia_Banderas",
    b"America/Chihuahua",
    b"Asia/Kabul",
    b"Asia/Urumqi",
    b"Africa/Johannesburg",
    b"ROK",
    b"America/Cambridge_Bay",
    b"Asia/Novosibirsk",
    b"Australia/Currie",
    b"America/St_Lucia",
    b"Asia/Oral",
    b"Europe/Rome",
    b"Pacific/Gambier",
    b"Indian/Chagos",
    b"Europe/Riga",
    b"Africa/Bamako",
    b"America/St_Vincent",
    b"Pacific/Pago_Pago",
    b"Australia/Lord_Howe",
    b"Australia/Lindeman",
    b"Europe/Nicosia",
    b"Libya",
    b"Pacific/Samoa",
    b"Australia/NSW",
    b"Egypt",
    b"Europe/Tirane",
    b"Australia/Victoria",
    b"Pacific/Noumea",
    b"Pacific/Marquesas",
    b"Europe/Athens",
    b"Europe/Helsinki",
    b"Asia/Jakarta",
    b"Zulu",
    b"Europe/Astrakhan",
    b"Africa/Banjul",
    b"Europe/Monaco",
    b"America/Hermosillo",
    b"America/Thunder_Bay",
    b"Pacific/Chatham",
    b"Australia/Broken_Hill",
    b"America/St_Johns",
    b"Europe/Bratislava",
    b"Brazil/Acre",
    b"Indian/Christmas",
    b"Pacific/Saipan",
    b"Factory",
    b"Antarctica/Syowa",
    b"Pacific/Enderbury",
    b"Europe/Prague",
    b"Asia/Kashgar",
    b"Atlantic/Cape_Verde",
    b"Antarctica/South_Pole",
    b"Antarctica/Davis",
    b"Europe/Vienna",
    b"Pacific/Pitcairn",
    b"Australia/Yancowinna",
    b"Asia/Barnaul",
    b"America/Dawson_Creek",
    b"Europe/Belgrade",
    b"Antarctica/DumontDUrville",
    b"America/Havana",
    b"America/St_Barthelemy",
    b"Europe/Paris",
    b"Europe/Sarajevo",
    b"America/Yellowknife",
    b"Pacific/Funafuti",
    b"Europe/London",
    b"Asia/Chungking",
    b"Pacific/Galapagos",
    b"America/Jujuy",
    b"Europe/Lisbon",
    b"Chile/Continental",
    b"Africa/Brazzaville",
    b"America/Shiprock",
    b"Europe/Volgograd",
    b"America/Halifax",
    b"Europe/Busingen",
    b"Pacific/Guam",
    b"Europe/Saratov",
    b"Atlantic/Stanley",
    b"Antarctica/Mawson",
    b"Antarctica/McMurdo",
    b"Pacific/Bougainville",
    b"Atlantic/Canary",
    b"Africa/Lusaka",
    b"Asia/Karachi",
    b"Antarctica/Macquarie",
    b"Europe/Kaliningrad",
    b"Asia/Kuching",
    b"America/Inuvik",
    b"US/Hawaii",
    b"Europe/Vilnius",
    b"America/Mexico_City",
    b"America/Los_Angeles",
    b"Antarctica/Palmer",
    b"Mexico/BajaNorte",
    b"Atlantic/St_Helena",
    b"Europe/Moscow",
    b"Africa/Conakry",
    b"Europe/Vatican",
    b"Antarctica/Troll",
    b"Europe/Warsaw",
    b"Asia/Jayapura",
    b"Asia/Kolkata",
    b"Europe/Oslo",
    b"Pacific/Majuro",
    b"Etc/Universal",
    b"America/Yakutat",
    b"Universal",
    b"Pacific/Kanton",
    b"Asia/Vladivostok",
    b"Africa/Kinshasa",
    b"Atlantic/Azores",
    b"Brazil/East",
    b"Europe/Tallinn",
    b"Europe/Dublin",
    b"Pacific/Kiritimati",
    b"Asia/Kathmandu",
    b"Pacific/Guadalcanal",
    b"Europe/Berlin",
    b"Pacific/Tahiti",
    b"America/Coyhaique",
    b"Mexico/BajaSur",
    b"Pacific/Palau",
    b"Australia/Hobart",
    b"Pacific/Port_Moresby",
    b"Australia/North",
    b"Europe/Amsterdam",
    b"Asia/Sakhalin",
    b"Pacific/Fiji",
    b"Asia/Ho_Chi_Minh",
    b"America/Adak",
    b"Etc/Zulu",
    b"Europe/Brussels",
    b"America/New_York",
    b"Africa/Khartoum",
    b"Pacific/Kosrae",
    b"Europe/Tiraspol",
    b"Europe/Budapest",
    b"America/St_Kitts",
    b"Asia/Novokuznetsk",
    b"Pacific/Honolulu",
    b"Pacific/Auckland",
    b"Asia/Kuala_Lumpur",
    b"Australia/South",
    b"Europe/Belfast",
    b"Asia/Phnom_Penh",
    b"Pacific/Wake",
    b"Pacific/Midway",
    b"Antarctica/Casey",
    b"Asia/Jerusalem",
    b"Asia/Ujung_Pandang",
    b"Kwajalein",
    b"America/Nuuk",
    b"America/La_Paz",
    b"Europe/Luxembourg",
    b"Asia/Tel_Aviv",
    b"Europe/Istanbul",
    b"Pacific/Kwajalein",
    b"Europe/San_Marino",
    b"Asia/Krasnoyarsk",
    b"Europe/Gibraltar",
    b"Pacific/Wallis",
    b"Pacific/Johnston",
    b"Pacific/Yap",
    b"Asia/Bangkok",
    b"Australia/Sydney",
    b"Africa/Windhoek",
    b"Pacific/Pohnpei",
    b"Asia/Omsk",
    b"Asia/Baghdad",
    b"Asia/Thimphu",
    b"Australia/Perth",
    b"Europe/Simferopol",
    b"Europe/Uzhgorod",
    b"Australia/LHI",
    b"Europe/Ljubljana",
    b"Greenwich",
    b"Europe/Isle_of_Man",
    b"America/Kralendijk",
    b"Pacific/Chuuk",
    b"Pacific/Fakaofo",
    b"Pacific/Norfolk",
    b"Turkey",
    b"Europe/Kiev",
    b"Europe/Kirov",
    b"Europe/Chisinau",
    b"Portugal",
    b"Atlantic/Reykjavik",
    b"Europe/Bucharest",
    b"Asia/Srednekolymsk",
    b"Europe/Zurich",
    b"Europe/Stockholm",
    b"Antarctica/Vostok",
    b"Asia/Riyadh",
    b"Europe/Vaduz",
    b"Europe/Jersey",
    b"Europe/Skopje",
    b"Pacific/Truk",
    b"Mexico/General",
    b"Etc/Greenwich",
    b"Europe/Guernsey",
    b"Hongkong",
    b"Europe/Ulyanovsk",
    b"Asia/Irkutsk",
    b"Europe/Kyiv",
    b"Asia/Bishkek",
    b"Europe/Minsk",
    b"Asia/Yakutsk",
];

pub(crate) const TIME_ZONES: [&TimeZoneRef<'static>; 599] = [
    &tzdata::EST,
    &tzdata::EST,
    &tzdata::EST_5_EDT,
    &tzdata::ASIA_MANILA,
    &tzdata::ASIA_MACAO,
    &tzdata::ASIA_AMMAN,
    &tzdata::INDIAN_MAHE,
    &tzdata::GMT,
    &tzdata::ASIA_NICOSIA,
    &tzdata::AMERICA_TORTOLA,
    &tzdata::INDIAN_MAYOTTE,
    &tzdata::GMT,
    &tzdata::GMT,
    &tzdata::GMT,
    &tzdata::AMERICA_ANTIGUA,
    &tzdata::AMERICA_MONTREAL,
    &tzdata::AMERICA_NORONHA,
    &tzdata::AMERICA_OJINAGA,
    &tzdata::AMERICA_DETROIT,
    &tzdata::AMERICA_NORTH_DAKOTA_CENTER,
    &tzdata::INDIAN_COMORO,
    &tzdata::AMERICA_RAINY_RIVER,
    &tzdata::AMERICA_BLANC_SABLON,
    &tzdata::INDIAN_ANTANANARIVO,
    &tzdata::ASIA_ANADYR,
    &tzdata::AMERICA_INDIANA_MARENGO,
    &tzdata::AMERICA_GUATEMALA,
    &tzdata::AMERICA_ARAGUAINA,
    &tzdata::AMERICA_GRENADA,
    &tzdata::IRAN,
    &tzdata::AMERICA_ARGENTINA_TUCUMAN,
    &tzdata::UCT,
    &tzdata::AMERICA_REGINA,
    &tzdata::AMERICA_GUADELOUPE,
    &tzdata::AFRICA_PORTO_NOVO,
    &tzdata::AFRICA_CEUTA,
    &tzdata::AFRICA_DAKAR,
    &tzdata::AFRICA_MONROVIA,
    &tzdata::AMERICA_BOGOTA,
    &tzdata::AFRICA_HARARE,
    &tzdata::AFRICA_ASMERA,
    &tzdata::ASIA_ASHGABAT,
    &tzdata::ASIA_QATAR,
    &tzdata::EGYPT,
    &tzdata::ASIA_MACAO,
    &tzdata::AMERICA_GODTHAB,
    &tzdata::AMERICA_KNOX_IN,
    &tzdata::AMERICA_INDIANA_PETERSBURG,
    &tzdata::AFRICA_MALABO,
    &tzdata::IRAN,
    &tzdata::ASIA_MUSCAT,
    &tzdata::AMERICA_SANTIAGO,
    &tzdata::AMERICA_ENSENADA,
    &tzdata::CST_6_CDT,
    &tzdata::AFRICA_CASABLANCA,
    &tzdata::PRC,
    &tzdata::AMERICA_CORDOBA,
    &tzdata::ASIA_AQTOBE,
    &tzdata::ASIA_DUBAI,
    &tzdata::AMERICA_RESOLUTE,
    &tzdata::AMERICA_SANTO_DOMINGO,
    &tzdata::INDIAN_REUNION,
    &tzdata::AMERICA_ASUNCION,
    &tzdata::AMERICA_FORT_WAYNE,
    &tzdata::ASIA_DACCA,
    &tzdata::INDIAN_MAURITIUS,
    &tzdata::AMERICA_EIRUNEPE,
    &tzdata::AFRICA_MBABANE,
    &tzdata::INDIAN_MALDIVES,
    &tzdata::ASIA_HEBRON,
    &tzdata::NZ_CHAT,
    &tzdata::AFRICA_FREETOWN,
    &tzdata::AMERICA_SCORESBYSUND,
    &tzdata::AMERICA_ARUBA,
    &tzdata::AMERICA_BOISE,
    &tzdata::AMERICA_CATAMARCA,
    &tzdata::ASIA_RANGOON,
    &tzdata::ASIA_BAHRAIN,
    &tzdata::AMERICA_BUENOS_AIRES,
    &tzdata::MST,
    &tzdata::AMERICA_CUIABA,
    &tzdata::AMERICA_INDIANA_VINCENNES,
    &tzdata::AMERICA_CURACAO,
    &tzdata::AMERICA_FORT_WAYNE,
    &tzdata::AMERICA_RECIFE,
    &tzdata::AMERICA_CORDOBA,
    &tzdata::AMERICA_FORT_WAYNE,
    &tzdata::AMERICA_BARBADOS,
    &tzdata::EST,
    &tzdata::GB,
    &tzdata::MST_7_MDT,
    &tzdata::AMERICA_ANGUILLA,
    &tzdata::AMERICA_ARGENTINA_SALTA,
    &tzdata::AMERICA_TEGUCIGALPA,
    &tzdata::EIRE,
    &tzdata::AMERICA_ARGENTINA_SAN_JUAN,
    &tzdata::ASIA_HOVD,
    &tzdata::AMERICA_EDMONTON,
    &tzdata::AMERICA_ARGENTINA_USHUAIA,
    &tzdata::AFRICA_MAPUTO,
    &tzdata::AMERICA_ARGENTINA_RIO_GALLEGOS,
    &tzdata::AMERICA_INDIANA_WINAMAC,
    &tzdata::ASIA_DILI,
    &tzdata::AMERICA_WHITEHORSE,
    &tzdata::AFRICA_MASERU,
    &tzdata::ASIA_CHITA,
    &tzdata::PST_8_PDT,
    &tzdata::JAPAN,
    &tzdata::AFRICA_MOGADISHU,
    &tzdata::AMERICA_BUENOS_AIRES,
    &tzdata::PACIFIC_PAGO_PAGO,
    &tzdata::AFRICA_ASMARA,
    &tzdata::AMERICA_MONTSERRAT,
    &tzdata::AFRICA_ASMERA,
    &tzdata::AMERICA_MANAGUA,
    &tzdata::AMERICA_NOME,
    &tzdata::AMERICA_CORDOBA,
    &tzdata::AMERICA_MONTEVIDEO,
    &tzdata::AMERICA_CATAMARCA,
    &tzdata::AMERICA_EDMONTON,
    &tzdata::AMERICA_MENOMINEE,
    &tzdata::AMERICA_MARTINIQUE,
    &tzdata::AMERICA_CATAMARCA,
    &tzdata::ASIA_AQTAU,
    &tzdata::JAMAICA,
    &tzdata::AMERICA_KRALENDIJK,
    &tzdata::AMERICA_MERIDA,
    &tzdata::AMERICA_MENDOZA,
    &tzdata::INDIAN_COCOS,
    &tzdata::AFRICA_ADDIS_ABABA,
    &tzdata::AMERICA_NORTH_DAKOTA_NEW_SALEM,
    &tzdata::ASIA_COLOMBO,
    &tzdata::MST_7_MDT,
    &tzdata::AMERICA_DOMINICA,
    &tzdata::AMERICA_KNOX_IN,
    &tzdata::AMERICA_RAINY_RIVER,
    &tzdata::AFRICA_ACCRA,
    &tzdata::EST,
    &tzdata::AMERICA_COSTA_RICA,
    &tzdata::ASIA_HO_CHI_MINH,
    &tzdata::AMERICA_CRESTON,
    &tzdata::ASIA_FAMAGUSTA,
    &tzdata::AMERICA_PORTO_ACRE,
    &tzdata::AMERICA_CARACAS,
    &tzdata::AMERICA_PARAMARIBO,
    &tzdata::AMERICA_MONTREAL,
    &tzdata::AMERICA_PORT_OF_SPAIN,
    &tzdata::AMERICA_PORTO_VELHO,
    &tzdata::GB,
    &tzdata::ICELAND,
    &tzdata::AMERICA_KRALENDIJK,
    &tzdata::AMERICA_PORT_AU_PRINCE,
    &tzdata::AMERICA_IQALUIT,
    &tzdata::AFRICA_LOME,
    &tzdata::AMERICA_MENDOZA,
    &tzdata::SINGAPORE,
    &tzdata::AMERICA_ARGENTINA_SAN_LUIS,
    &tzdata::ASIA_KUWAIT,
    &tzdata::AMERICA_CANCUN,
    &tzdata::AFRICA_NDJAMENA,
    &tzdata::CST_6_CDT,
    &tzdata::AFRICA_LIBREVILLE,
    &tzdata::AMERICA_DAWSON,
    &tzdata::MST,
    &tzdata::PRC,
    &tzdata::NZ,
    &tzdata::AMERICA_BELIZE,
    &tzdata::MST_7_MDT,
    &tzdata::AMERICA_MATAMOROS,
    &tzdata::AMERICA_SANTAREM,
    &tzdata::AMERICA_MANAUS,
    &tzdata::UCT,
    &tzdata::ASIA_CHOIBALSAN,
    &tzdata::ASIA_CHOIBALSAN,
    &tzdata::AMERICA_NASSAU,
    &tzdata::AMERICA_MIQUELON,
    &tzdata::ASIA_QOSTANAY,
    &tzdata::AFRICA_DJIBOUTI,
    &tzdata::ASIA_CALCUTTA,
    &tzdata::AMERICA_ENSENADA,
    &tzdata::ASIA_BEIRUT,
    &tzdata::ROC,
    &tzdata::AMERICA_ARGENTINA_LA_RIOJA,
    &tzdata::ASIA_DAMASCUS,
    &tzdata::AFRICA_DOUALA,
    &tzdata::AMERICA_JUNEAU,
    &tzdata::AMERICA_HALIFAX,
    &tzdata::AMERICA_KRALENDIJK,
    &tzdata::ASIA_QYZYLORDA,
    &tzdata::EET,
    &tzdata::AMERICA_MACEIO,
    &tzdata::AMERICA_MONCTON,
    &tzdata::ASIA_MAKASSAR,
    &tzdata::AMERICA_PUNTA_ARENAS,
    &tzdata::W_SU,
    &tzdata::ICELAND,
    &tzdata::AMERICA_ANCHORAGE,
    &tzdata::CUBA,
    &tzdata::ASIA_UST_NERA,
    &tzdata::AMERICA_THULE,
    &tzdata::ASIA_RANGOON,
    &tzdata::AMERICA_ENSENADA,
    &tzdata::ASIA_KAMCHATKA,
    &tzdata::AFRICA_BANGUI,
    &tzdata::AMERICA_LOUISVILLE,
    &tzdata::ASIA_ADEN,
    &tzdata::AMERICA_GUYANA,
    &tzdata::AFRICA_LUANDA,
    &tzdata::AMERICA_PORTO_ACRE,
    &tzdata::ASIA_BRUNEI,
    &tzdata::AMERICA_FORTALEZA,
    &tzdata::AMERICA_FORT_WAYNE,
    &tzdata::AMERICA_IQALUIT,
    &tzdata::AMERICA_BOA_VISTA,
    &tzdata::AMERICA_FORT_NELSON,
    &tzdata::JAMAICA,
    &tzdata::ASIA_YEREVAN,
    &tzdata::PRC,
    &tzdata::AMERICA_VANCOUVER,
    &tzdata::ISRAEL,
    &tzdata::AFRICA_BISSAU,
    &tzdata::INDIAN_KERGUELEN,
    &tzdata::LIBYA,
    &tzdata::ICELAND,
    &tzdata::GMT,
    &tzdata::POLAND,
    &tzdata::GMT,
    &tzdata::SINGAPORE,
    &tzdata::GMT,
    &tzdata::AFRICA_SAO_TOME,
    &tzdata::AMERICA_RAINY_RIVER,
    &tzdata::UCT,
    &tzdata::ETC_GMT_MINUS_10,
    &tzdata::GMT,
    &tzdata::ETC_GMT_MINUS_1,
    &tzdata::ETC_GMT_MINUS_11,
    &tzdata::ETC_GMT_PLUS_10,
    &tzdata::ETC_GMT_MINUS_8,
    &tzdata::ETC_GMT_PLUS_1,
    &tzdata::ETC_GMT_PLUS_11,
    &tzdata::AMERICA_LIMA,
    &tzdata::ETC_GMT_PLUS_8,
    &tzdata::AMERICA_METLAKATLA,
    &tzdata::AMERICA_MONTREAL,
    &tzdata::ETC_GMT_MINUS_12,
    &tzdata::PACIFIC_EFATE,
    &tzdata::AFRICA_ALGIERS,
    &tzdata::AFRICA_BLANTYRE,
    &tzdata::ETC_GMT_PLUS_12,
    &tzdata::PACIFIC_TARAWA,
    &tzdata::ETC_GMT_MINUS_7,
    &tzdata::PACIFIC_APIA,
    &tzdata::ETC_GMT_MINUS_2,
    &tzdata::AFRICA_LAGOS,
    &tzdata::ETC_GMT_MINUS_14,
    &tzdata::ETC_GMT_MINUS_13,
    &tzdata::ETC_GMT_PLUS_7,
    &tzdata::ETC_GMT_PLUS_2,
    &tzdata::CET,
    &tzdata::ETC_GMT_MINUS_6,
    &tzdata::EST_5_EDT,
    &tzdata::ETC_GMT_MINUS_5,
    &tzdata::AMERICA_CAYENNE,
    &tzdata::PACIFIC_RAROTONGA,
    &tzdata::ETC_GMT_PLUS_6,
    &tzdata::ETC_GMT_PLUS_5,
    &tzdata::AMERICA_BELEM,
    &tzdata::ETC_GMT_MINUS_9,
    &tzdata::AMERICA_SAO_PAULO,
    &tzdata::ETC_GMT_MINUS_4,
    &tzdata::AUSTRALIA_CURRIE,
    &tzdata::ETC_GMT_MINUS_3,
    &tzdata::ETC_GMT_PLUS_9,
    &tzdata::ETC_GMT_PLUS_4,
    &tzdata::ASIA_CHOIBALSAN,
    &tzdata::ETC_GMT_PLUS_3,
    &tzdata::AFRICA_TUNIS,
    &tzdata::AMERICA_GOOSE_BAY,
    &tzdata::AMERICA_DANMARKSHAVN,
    &tzdata::AMERICA_EL_SALVADOR,
    &tzdata::AUSTRALIA_DARWIN,
    &tzdata::AUSTRALIA_ADELAIDE,
    &tzdata::AMERICA_SWIFT_CURRENT,
    &tzdata::PORTUGAL,
    &tzdata::HST,
    &tzdata::PST_8_PDT,
    &tzdata::AUSTRALIA_BRISBANE,
    &tzdata::MST,
    &tzdata::ASIA_PONTIANAK,
    &tzdata::AMERICA_GLACE_BAY,
    &tzdata::CHILE_EASTER_ISLAND,
    &tzdata::ASIA_SAMARKAND,
    &tzdata::CET,
    &tzdata::ASIA_DACCA,
    &tzdata::AMERICA_MONTERREY,
    &tzdata::AMERICA_MAZATLAN,
    &tzdata::AMERICA_ADAK,
    &tzdata::ROC,
    &tzdata::ASIA_GAZA,
    &tzdata::ASIA_VIENTIANE,
    &tzdata::AMERICA_ATIKOKAN,
    &tzdata::AFRICA_KAMPALA,
    &tzdata::ASIA_ATYRAU,
    &tzdata::ASIA_MAGADAN,
    &tzdata::AMERICA_MONTREAL,
    &tzdata::AMERICA_RANKIN_INLET,
    &tzdata::AMERICA_LOUISVILLE,
    &tzdata::AMERICA_KENTUCKY_MONTICELLO,
    &tzdata::ROK,
    &tzdata::AMERICA_DETROIT,
    &tzdata::CHILE_EASTER_ISLAND,
    &tzdata::CST_6_CDT,
    &tzdata::PACIFIC_NIUE,
    &tzdata::AFRICA_OUAGADOUGOU,
    &tzdata::ASIA_BAKU,
    &tzdata::AMERICA_ST_THOMAS,
    &tzdata::ATLANTIC_FAEROE,
    &tzdata::ASIA_DUSHANBE,
    &tzdata::ARCTIC_LONGYEARBYEN,
    &tzdata::ATLANTIC_FAEROE,
    &tzdata::ATLANTIC_BERMUDA,
    &tzdata::PRC,
    &tzdata::ATLANTIC_MADEIRA,
    &tzdata::AMERICA_CAMPO_GRANDE,
    &tzdata::AMERICA_ADAK,
    &tzdata::ASIA_ASHGABAT,
    &tzdata::AMERICA_NORTH_DAKOTA_BEULAH,
    &tzdata::AFRICA_KIGALI,
    &tzdata::AUSTRALIA_EUCLA,
    &tzdata::AMERICA_ST_JOHNS,
    &tzdata::AMERICA_KNOX_IN,
    &tzdata::AMERICA_GRAND_TURK,
    &tzdata::ASIA_PYONGYANG,
    &tzdata::AMERICA_BAHIA,
    &tzdata::AMERICA_CIUDAD_JUAREZ,
    &tzdata::AFRICA_NIAMEY,
    &tzdata::AFRICA_NOUAKCHOTT,
    &tzdata::AMERICA_JUJUY,
    &tzdata::AMERICA_WHITEHORSE,
    &tzdata::AUSTRALIA_ACT,
    &tzdata::AFRICA_BUJUMBURA,
    &tzdata::AUSTRALIA_BRISBANE,
    &tzdata::PACIFIC_TONGATAPU,
    &tzdata::AUSTRALIA_ACT,
    &tzdata::ASIA_YEKATERINBURG,
    &tzdata::AMERICA_INDIANA_TELL_CITY,
    &tzdata::HONGKONG,
    &tzdata::AMERICA_SITKA,
    &tzdata::EUROPE_MALTA,
    &tzdata::AMERICA_INDIANA_VEVAY,
    &tzdata::EUROPE_SAMARA,
    &tzdata::AMERICA_GUAYAQUIL,
    &tzdata::ASIA_KHANDYGA,
    &tzdata::EUROPE_SOFIA,
    &tzdata::AMERICA_NORONHA,
    &tzdata::AFRICA_GABORONE,
    &tzdata::AFRICA_DAR_ES_SALAAM,
    &tzdata::AUSTRALIA_PERTH,
    &tzdata::EUROPE_HELSINKI,
    &tzdata::ASIA_TASHKENT,
    &tzdata::MST_7_MDT,
    &tzdata::EUROPE_KIEV,
    &tzdata::ASIA_KATHMANDU,
    &tzdata::EUROPE_COPENHAGEN,
    &tzdata::JAPAN,
    &tzdata::ASIA_ALMATY,
    &tzdata::ASIA_TBILISI,
    &tzdata::AMERICA_CAYMAN,
    &tzdata::EUROPE_MADRID,
    &tzdata::AMERICA_KRALENDIJK,
    &tzdata::UCT,
    &tzdata::EUROPE_ANDORRA,
    &tzdata::AMERICA_MANAUS,
    &tzdata::EUROPE_ZAGREB,
    &tzdata::PACIFIC_NAURU,
    &tzdata::TURKEY,
    &tzdata::ARCTIC_LONGYEARBYEN,
    &tzdata::AFRICA_LUBUMBASHI,
    &tzdata::PACIFIC_GUADALCANAL,
    &tzdata::AUSTRALIA_MELBOURNE,
    &tzdata::ATLANTIC_SOUTH_GEORGIA,
    &tzdata::AMERICA_VANCOUVER,
    &tzdata::EUROPE_BELGRADE,
    &tzdata::ASIA_THIMBU,
    &tzdata::AFRICA_JUBA,
    &tzdata::AMERICA_ANCHORAGE,
    &tzdata::AFRICA_EL_AAIUN,
    &tzdata::ANTARCTICA_ROTHERA,
    &tzdata::AMERICA_REGINA,
    &tzdata::ASIA_TOMSK,
    &tzdata::AMERICA_BAHIA_BANDERAS,
    &tzdata::AMERICA_CHIHUAHUA,
    &tzdata::ASIA_KABUL,
    &tzdata::ASIA_KASHGAR,
    &tzdata::AFRICA_JOHANNESBURG,
    &tzdata::ROK,
    &tzdata::AMERICA_CAMBRIDGE_BAY,
    &tzdata::ASIA_NOVOSIBIRSK,
    &tzdata::AUSTRALIA_CURRIE,
    &tzdata::AMERICA_ST_LUCIA,
    &tzdata::ASIA_ORAL,
    &tzdata::EUROPE_ROME,
    &tzdata::PACIFIC_GAMBIER,
    &tzdata::INDIAN_CHAGOS,
    &tzdata::EUROPE_RIGA,
    &tzdata::AFRICA_BAMAKO,
    &tzdata::AMERICA_ST_VINCENT,
    &tzdata::PACIFIC_PAGO_PAGO,
    &tzdata::AUSTRALIA_LHI,
    &tzdata::AUSTRALIA_LINDEMAN,
    &tzdata::ASIA_NICOSIA,
    &tzdata::LIBYA,
    &tzdata::PACIFIC_PAGO_PAGO,
    &tzdata::AUSTRALIA_ACT,
    &tzdata::EGYPT,
    &tzdata::EUROPE_TIRANE,
    &tzdata::AUSTRALIA_MELBOURNE,
    &tzdata::PACIFIC_NOUMEA,
    &tzdata::PACIFIC_MARQUESAS,
    &tzdata::EET,
    &tzdata::EUROPE_HELSINKI,
    &tzdata::ASIA_JAKARTA,
    &tzdata::UCT,
    &tzdata::EUROPE_ASTRAKHAN,
    &tzdata::AFRICA_BANJUL,
    &tzdata::EUROPE_MONACO,
    &tzdata::AMERICA_HERMOSILLO,
    &tzdata::AMERICA_MONTREAL,
    &tzdata::NZ_CHAT,
    &tzdata::AUSTRALIA_BROKEN_HILL,
    &tzdata::AMERICA_ST_JOHNS,
    &tzdata::EUROPE_BRATISLAVA,
    &tzdata::AMERICA_PORTO_ACRE,
    &tzdata::INDIAN_CHRISTMAS,
    &tzdata::PACIFIC_SAIPAN,
    &tzdata::FACTORY,
    &tzdata::ANTARCTICA_SYOWA,
    &tzdata::PACIFIC_ENDERBURY,
    &tzdata::EUROPE_BRATISLAVA,
    &tzdata::ASIA_KASHGAR,
    &tzdata::ATLANTIC_CAPE_VERDE,
    &tzdata::NZ,
    &tzdata::ANTARCTICA_DAVIS,
    &tzdata::EUROPE_VIENNA,
    &tzdata::PACIFIC_PITCAIRN,
    &tzdata::AUSTRALIA_BROKEN_HILL,
    &tzdata::ASIA_BARNAUL,
    &tzdata::AMERICA_DAWSON_CREEK,
    &tzdata::EUROPE_BELGRADE,
    &tzdata::ANTARCTICA_DUMONT_D_URVILLE,
    &tzdata::CUBA,
    &tzdata::AMERICA_KRALENDIJK,
    &tzdata::EUROPE_PARIS,
    &tzdata::EUROPE_SARAJEVO,
    &tzdata::AMERICA_EDMONTON,
    &tzdata::PACIFIC_FUNAFUTI,
    &tzdata::GB,
    &tzdata::PRC,
    &tzdata::PACIFIC_GALAPAGOS,
    &tzdata::AMERICA_JUJUY,
    &tzdata::PORTUGAL,
    &tzdata::AMERICA_SANTIAGO,
    &tzdata::AFRICA_BRAZZAVILLE,
    &tzdata::MST_7_MDT,
    &tzdata::EUROPE_VOLGOGRAD,
    &tzdata::AMERICA_HALIFAX,
    &tzdata::EUROPE_BUSINGEN,
    &tzdata::PACIFIC_GUAM,
    &tzdata::EUROPE_SARATOV,
    &tzdata::ATLANTIC_STANLEY,
    &tzdata::ANTARCTICA_MAWSON,
    &tzdata::ANTARCTICA_MC_MURDO,
    &tzdata::PACIFIC_BOUGAINVILLE,
    &tzdata::ATLANTIC_CANARY,
    &tzdata::AFRICA_LUSAKA,
    &tzdata::ASIA_KARACHI,
    &tzdata::ANTARCTICA_MACQUARIE,
    &tzdata::EUROPE_KALININGRAD,
    &tzdata::ASIA_KUCHING,
    &tzdata::AMERICA_INUVIK,
    &tzdata::HST,
    &tzdata::EUROPE_VILNIUS,
    &tzdata::AMERICA_MEXICO_CITY,
    &tzdata::PST_8_PDT,
    &tzdata::ANTARCTICA_PALMER,
    &tzdata::AMERICA_ENSENADA,
    &tzdata::ATLANTIC_ST_HELENA,
    &tzdata::W_SU,
    &tzdata::AFRICA_CONAKRY,
    &tzdata::EUROPE_ROME,
    &tzdata::ANTARCTICA_TROLL,
    &tzdata::POLAND,
    &tzdata::ASIA_JAYAPURA,
    &tzdata::ASIA_CALCUTTA,
    &tzdata::EUROPE_OSLO,
    &tzdata::PACIFIC_MAJURO,
    &tzdata::UCT,
    &tzdata::AMERICA_YAKUTAT,
    &tzdata::UCT,
    &tzdata::PACIFIC_ENDERBURY,
    &tzdata::ASIA_VLADIVOSTOK,
    &tzdata::AFRICA_KINSHASA,
    &tzdata::ATLANTIC_AZORES,
    &tzdata::AMERICA_SAO_PAULO,
    &tzdata::EUROPE_TALLINN,
    &tzdata::EIRE,
    &tzdata::PACIFIC_KIRITIMATI,
    &tzdata::ASIA_KATHMANDU,
    &tzdata::PACIFIC_GUADALCANAL,
    &tzdata::ARCTIC_LONGYEARBYEN,
    &tzdata::PACIFIC_TAHITI,
    &tzdata::AMERICA_COYHAIQUE,
    &tzdata::AMERICA_MAZATLAN,
    &tzdata::PACIFIC_PALAU,
    &tzdata::AUSTRALIA_CURRIE,
    &tzdata::PACIFIC_PORT_MORESBY,
    &tzdata::AUSTRALIA_DARWIN,
    &tzdata::EUROPE_AMSTERDAM,
    &tzdata::ASIA_SAKHALIN,
    &tzdata::PACIFIC_FIJI,
    &tzdata::ASIA_HO_CHI_MINH,
    &tzdata::AMERICA_ADAK,
    &tzdata::UCT,
    &tzdata::CET,
    &tzdata::EST_5_EDT,
    &tzdata::AFRICA_KHARTOUM,
    &tzdata::PACIFIC_KOSRAE,
    &tzdata::EUROPE_CHISINAU,
    &tzdata::EUROPE_BUDAPEST,
    &tzdata::AMERICA_ST_KITTS,
    &tzdata::ASIA_NOVOKUZNETSK,
    &tzdata::HST,
    &tzdata::NZ,
    &tzdata::ASIA_KUALA_LUMPUR,
    &tzdata::AUSTRALIA_ADELAIDE,
    &tzdata::GB,
    &tzdata::ASIA_PHNOM_PENH,
    &tzdata::PACIFIC_WAKE,
    &tzdata::PACIFIC_MIDWAY,
    &tzdata::ANTARCTICA_CASEY,
    &tzdata::ISRAEL,
    &tzdata::ASIA_MAKASSAR,
    &tzdata::KWAJALEIN,
    &tzdata::AMERICA_GODTHAB,
    &tzdata::AMERICA_LA_PAZ,
    &tzdata::EUROPE_LUXEMBOURG,
    &tzdata::ISRAEL,
    &tzdata::TURKEY,
    &tzdata::KWAJALEIN,
    &tzdata::EUROPE_ROME,
    &tzdata::ASIA_KRASNOYARSK,
    &tzdata::EUROPE_GIBRALTAR,
    &tzdata::PACIFIC_WALLIS,
    &tzdata::HST,
    &tzdata::PACIFIC_PORT_MORESBY,
    &tzdata::ASIA_BANGKOK,
    &tzdata::AUSTRALIA_ACT,
    &tzdata::AFRICA_WINDHOEK,
    &tzdata::PACIFIC_POHNPEI,
    &tzdata::ASIA_OMSK,
    &tzdata::ASIA_BAGHDAD,
    &tzdata::ASIA_THIMBU,
    &tzdata::AUSTRALIA_PERTH,
    &tzdata::EUROPE_SIMFEROPOL,
    &tzdata::EUROPE_KIEV,
    &tzdata::AUSTRALIA_LHI,
    &tzdata::EUROPE_LJUBLJANA,
    &tzdata::GMT,
    &tzdata::EUROPE_ISLE_OF_MAN,
    &tzdata::AMERICA_KRALENDIJK,
    &tzdata::PACIFIC_CHUUK,
    &tzdata::PACIFIC_FAKAOFO,
    &tzdata::PACIFIC_NORFOLK,
    &tzdata::TURKEY,
    &tzdata::EUROPE_KIEV,
    &tzdata::EUROPE_KIROV,
    &tzdata::EUROPE_CHISINAU,
    &tzdata::PORTUGAL,
    &tzdata::ATLANTIC_REYKJAVIK,
    &tzdata::EUROPE_BUCHAREST,
    &tzdata::ASIA_SREDNEKOLYMSK,
    &tzdata::EUROPE_BUSINGEN,
    &tzdata::EUROPE_STOCKHOLM,
    &tzdata::ANTARCTICA_VOSTOK,
    &tzdata::ASIA_RIYADH,
    &tzdata::EUROPE_VADUZ,
    &tzdata::EUROPE_JERSEY,
    &tzdata::EUROPE_SKOPJE,
    &tzdata::PACIFIC_PORT_MORESBY,
    &tzdata::AMERICA_MEXICO_CITY,
    &tzdata::GMT,
    &tzdata::EUROPE_GUERNSEY,
    &tzdata::HONGKONG,
    &tzdata::EUROPE_ULYANOVSK,
    &tzdata::ASIA_IRKUTSK,
    &tzdata::EUROPE_KIEV,
    &tzdata::ASIA_BISHKEK,
    &tzdata::EUROPE_MINSK,
    &tzdata::ASIA_YAKUTSK,
];

pub(crate) const RAW_TIME_ZONES: [&[u8]; 599] = [
    raw_tzdata::EST,
    raw_tzdata::EST,
    raw_tzdata::EST_5_EDT,
    raw_tzdata::ASIA_MANILA,
    raw_tzdata::ASIA_MACAO,
    raw_tzdata::ASIA_AMMAN,
    raw_tzdata::INDIAN_MAHE,
    raw_tzdata::GMT,
    raw_tzdata::ASIA_NICOSIA,
    raw_tzdata::AMERICA_TORTOLA,
    raw_tzdata::INDIAN_MAYOTTE,
    raw_tzdata::GMT,
    raw_tzdata::GMT,
    raw_tzdata::GMT,
    raw_tzdata::AMERICA_ANTIGUA,
    raw_tzdata::AMERICA_MONTREAL,
    raw_tzdata::AMERICA_NORONHA,
    raw_tzdata::AMERICA_OJINAGA,
    raw_tzdata::AMERICA_DETROIT,
    raw_tzdata::AMERICA_NORTH_DAKOTA_CENTER,
    raw_tzdata::INDIAN_COMORO,
    raw_tzdata::AMERICA_RAINY_RIVER,
    raw_tzdata::AMERICA_BLANC_SABLON,
    raw_tzdata::INDIAN_ANTANANARIVO,
    raw_tzdata::ASIA_ANADYR,
    raw_tzdata::AMERICA_INDIANA_MARENGO,
    raw_tzdata::AMERICA_GUATEMALA,
    raw_tzdata::AMERICA_ARAGUAINA,
    raw_tzdata::AMERICA_GRENADA,
    raw_tzdata::IRAN,
    raw_tzdata::AMERICA_ARGENTINA_TUCUMAN,
    raw_tzdata::UCT,
    raw_tzdata::AMERICA_REGINA,
    raw_tzdata::AMERICA_GUADELOUPE,
    raw_tzdata::AFRICA_PORTO_NOVO,
    raw_tzdata::AFRICA_CEUTA,
    raw_tzdata::AFRICA_DAKAR,
    raw_tzdata::AFRICA_MONROVIA,
    raw_tzdata::AMERICA_BOGOTA,
    raw_tzdata::AFRICA_HARARE,
    raw_tzdata::AFRICA_ASMERA,
    raw_tzdata::ASIA_ASHGABAT,
    raw_tzdata::ASIA_QATAR,
    raw_tzdata::EGYPT,
    raw_tzdata::ASIA_MACAO,
    raw_tzdata::AMERICA_GODTHAB,
    raw_tzdata::AMERICA_KNOX_IN,
    raw_tzdata::AMERICA_INDIANA_PETERSBURG,
    raw_tzdata::AFRICA_MALABO,
    raw_tzdata::IRAN,
    raw_tzdata::ASIA_MUSCAT,
    raw_tzdata::AMERICA_SANTIAGO,
    raw_tzdata::AMERICA_ENSENADA,
    raw_tzdata::CST_6_CDT,
    raw_tzdata::AFRICA_CASABLANCA,
    raw_tzdata::PRC,
    raw_tzdata::AMERICA_CORDOBA,
    raw_tzdata::ASIA_AQTOBE,
    raw_tzdata::ASIA_DUBAI,
    raw_tzdata::AMERICA_RESOLUTE,
    raw_tzdata::AMERICA_SANTO_DOMINGO,
    raw_tzdata::INDIAN_REUNION,
    raw_tzdata::AMERICA_ASUNCION,
    raw_tzdata::AMERICA_FORT_WAYNE,
    raw_tzdata::ASIA_DACCA,
    raw_tzdata::INDIAN_MAURITIUS,
    raw_tzdata::AMERICA_EIRUNEPE,
    raw_tzdata::AFRICA_MBABANE,
    raw_tzdata::INDIAN_MALDIVES,
    raw_tzdata::ASIA_HEBRON,
    raw_tzdata::NZ_CHAT,
    raw_tzdata::AFRICA_FREETOWN,
    raw_tzdata::AMERICA_SCORESBYSUND,
    raw_tzdata::AMERICA_ARUBA,
    raw_tzdata::AMERICA_BOISE,
    raw_tzdata::AMERICA_CATAMARCA,
    raw_tzdata::ASIA_RANGOON,
    raw_tzdata::ASIA_BAHRAIN,
    raw_tzdata::AMERICA_BUENOS_AIRES,
    raw_tzdata::MST,
    raw_tzdata::AMERICA_CUIABA,
    raw_tzdata::AMERICA_INDIANA_VINCENNES,
    raw_tzdata::AMERICA_CURACAO,
    raw_tzdata::AMERICA_FORT_WAYNE,
    raw_tzdata::AMERICA_RECIFE,
    raw_tzdata::AMERICA_CORDOBA,
    raw_tzdata::AMERICA_FORT_WAYNE,
    raw_tzdata::AMERICA_BARBADOS,
    raw_tzdata::EST,
    raw_tzdata::GB,
    raw_tzdata::MST_7_MDT,
    raw_tzdata::AMERICA_ANGUILLA,
    raw_tzdata::AMERICA_ARGENTINA_SALTA,
    raw_tzdata::AMERICA_TEGUCIGALPA,
    raw_tzdata::EIRE,
    raw_tzdata::AMERICA_ARGENTINA_SAN_JUAN,
    raw_tzdata::ASIA_HOVD,
    raw_tzdata::AMERICA_EDMONTON,
    raw_tzdata::AMERICA_ARGENTINA_USHUAIA,
    raw_tzdata::AFRICA_MAPUTO,
    raw_tzdata::AMERICA_ARGENTINA_RIO_GALLEGOS,
    raw_tzdata::AMERICA_INDIANA_WINAMAC,
    raw_tzdata::ASIA_DILI,
    raw_tzdata::AMERICA_WHITEHORSE,
    raw_tzdata::AFRICA_MASERU,
    raw_tzdata::ASIA_CHITA,
    raw_tzdata::PST_8_PDT,
    raw_tzdata::JAPAN,
    raw_tzdata::AFRICA_MOGADISHU,
    raw_tzdata::AMERICA_BUENOS_AIRES,
    raw_tzdata::PACIFIC_PAGO_PAGO,
    raw_tzdata::AFRICA_ASMARA,
    raw_tzdata::AMERICA_MONTSERRAT,
    raw_tzdata::AFRICA_ASMERA,
    raw_tzdata::AMERICA_MANAGUA,
    raw_tzdata::AMERICA_NOME,
    raw_tzdata::AMERICA_CORDOBA,
    raw_tzdata::AMERICA_MONTEVIDEO,
    raw_tzdata::AMERICA_CATAMARCA,
    raw_tzdata::AMERICA_EDMONTON,
    raw_tzdata::AMERICA_MENOMINEE,
    raw_tzdata::AMERICA_MARTINIQUE,
    raw_tzdata::AMERICA_CATAMARCA,
    raw_tzdata::ASIA_AQTAU,
    raw_tzdata::JAMAICA,
    raw_tzdata::AMERICA_KRALENDIJK,
    raw_tzdata::AMERICA_MERIDA,
    raw_tzdata::AMERICA_MENDOZA,
    raw_tzdata::INDIAN_COCOS,
    raw_tzdata::AFRICA_ADDIS_ABABA,
    raw_tzdata::AMERICA_NORTH_DAKOTA_NEW_SALEM,
    raw_tzdata::ASIA_COLOMBO,
    raw_tzdata::MST_7_MDT,
    raw_tzdata::AMERICA_DOMINICA,
    raw_tzdata::AMERICA_KNOX_IN,
    raw_tzdata::AMERICA_RAINY_RIVER,
    raw_tzdata::AFRICA_ACCRA,
    raw_tzdata::EST,
    raw_tzdata::AMERICA_COSTA_RICA,
    raw_tzdata::ASIA_HO_CHI_MINH,
    raw_tzdata::AMERICA_CRESTON,
    raw_tzdata::ASIA_FAMAGUSTA,
    raw_tzdata::AMERICA_PORTO_ACRE,
    raw_tzdata::AMERICA_CARACAS,
    raw_tzdata::AMERICA_PARAMARIBO,
    raw_tzdata::AMERICA_MONTREAL,
    raw_tzdata::AMERICA_PORT_OF_SPAIN,
    raw_tzdata::AMERICA_PORTO_VELHO,
    raw_tzdata::GB,
    raw_tzdata::ICELAND,
    raw_tzdata::AMERICA_KRALENDIJK,
    raw_tzdata::AMERICA_PORT_AU_PRINCE,
    raw_tzdata::AMERICA_IQALUIT,
    raw_tzdata::AFRICA_LOME,
    raw_tzdata::AMERICA_MENDOZA,
    raw_tzdata::SINGAPORE,
    raw_tzdata::AMERICA_ARGENTINA_SAN_LUIS,
    raw_tzdata::ASIA_KUWAIT,
    raw_tzdata::AMERICA_CANCUN,
    raw_tzdata::AFRICA_NDJAMENA,
    raw_tzdata::CST_6_CDT,
    raw_tzdata::AFRICA_LIBREVILLE,
    raw_tzdata::AMERICA_DAWSON,
    raw_tzdata::MST,
    raw_tzdata::PRC,
    raw_tzdata::NZ,
    raw_tzdata::AMERICA_BELIZE,
    raw_tzdata::MST_7_MDT,
    raw_tzdata::AMERICA_MATAMOROS,
    raw_tzdata::AMERICA_SANTAREM,
    raw_tzdata::AMERICA_MANAUS,
    raw_tzdata::UCT,
    raw_tzdata::ASIA_CHOIBALSAN,
    raw_tzdata::ASIA_CHOIBALSAN,
    raw_tzdata::AMERICA_NASSAU,
    raw_tzdata::AMERICA_MIQUELON,
    raw_tzdata::ASIA_QOSTANAY,
    raw_tzdata::AFRICA_DJIBOUTI,
    raw_tzdata::ASIA_CALCUTTA,
    raw_tzdata::AMERICA_ENSENADA,
    raw_tzdata::ASIA_BEIRUT,
    raw_tzdata::ROC,
    raw_tzdata::AMERICA_ARGENTINA_LA_RIOJA,
    raw_tzdata::ASIA_DAMASCUS,
    raw_tzdata::AFRICA_DOUALA,
    raw_tzdata::AMERICA_JUNEAU,
    raw_tzdata::AMERICA_HALIFAX,
    raw_tzdata::AMERICA_KRALENDIJK,
    raw_tzdata::ASIA_QYZYLORDA,
    raw_tzdata::EET,
    raw_tzdata::AMERICA_MACEIO,
    raw_tzdata::AMERICA_MONCTON,
    raw_tzdata::ASIA_MAKASSAR,
    raw_tzdata::AMERICA_PUNTA_ARENAS,
    raw_tzdata::W_SU,
    raw_tzdata::ICELAND,
    raw_tzdata::AMERICA_ANCHORAGE,
    raw_tzdata::CUBA,
    raw_tzdata::ASIA_UST_NERA,
    raw_tzdata::AMERICA_THULE,
    raw_tzdata::ASIA_RANGOON,
    raw_tzdata::AMERICA_ENSENADA,
    raw_tzdata::ASIA_KAMCHATKA,
    raw_tzdata::AFRICA_BANGUI,
    raw_tzdata::AMERICA_LOUISVILLE,
    raw_tzdata::ASIA_ADEN,
    raw_tzdata::AMERICA_GUYANA,
    raw_tzdata::AFRICA_LUANDA,
    raw_tzdata::AMERICA_PORTO_ACRE,
    raw_tzdata::ASIA_BRUNEI,
    raw_tzdata::AMERICA_FORTALEZA,
    raw_tzdata::AMERICA_FORT_WAYNE,
    raw_tzdata::AMERICA_IQALUIT,
    raw_tzdata::AMERICA_BOA_VISTA,
    raw_tzdata::AMERICA_FORT_NELSON,
    raw_tzdata::JAMAICA,
    raw_tzdata::ASIA_YEREVAN,
    raw_tzdata::PRC,
    raw_tzdata::AMERICA_VANCOUVER,
    raw_tzdata::ISRAEL,
    raw_tzdata::AFRICA_BISSAU,
    raw_tzdata::INDIAN_KERGUELEN,
    raw_tzdata::LIBYA,
    raw_tzdata::ICELAND,
    raw_tzdata::GMT,
    raw_tzdata::POLAND,
    raw_tzdata::GMT,
    raw_tzdata::SINGAPORE,
    raw_tzdata::GMT,
    raw_tzdata::AFRICA_SAO_TOME,
    raw_tzdata::AMERICA_RAINY_RIVER,
    raw_tzdata::UCT,
    raw_tzdata::ETC_GMT_MINUS_10,
    raw_tzdata::GMT,
    raw_tzdata::ETC_GMT_MINUS_1,
    raw_tzdata::ETC_GMT_MINUS_11,
    raw_tzdata::ETC_GMT_PLUS_10,
    raw_tzdata::ETC_GMT_MINUS_8,
    raw_tzdata::ETC_GMT_PLUS_1,
    raw_tzdata::ETC_GMT_PLUS_11,
    raw_tzdata::AMERICA_LIMA,
    raw_tzdata::ETC_GMT_PLUS_8,
    raw_tzdata::AMERICA_METLAKATLA,
    raw_tzdata::AMERICA_MONTREAL,
    raw_tzdata::ETC_GMT_MINUS_12,
    raw_tzdata::PACIFIC_EFATE,
    raw_tzdata::AFRICA_ALGIERS,
    raw_tzdata::AFRICA_BLANTYRE,
    raw_tzdata::ETC_GMT_PLUS_12,
    raw_tzdata::PACIFIC_TARAWA,
    raw_tzdata::ETC_GMT_MINUS_7,
    raw_tzdata::PACIFIC_APIA,
    raw_tzdata::ETC_GMT_MINUS_2,
    raw_tzdata::AFRICA_LAGOS,
    raw_tzdata::ETC_GMT_MINUS_14,
    raw_tzdata::ETC_GMT_MINUS_13,
    raw_tzdata::ETC_GMT_PLUS_7,
    raw_tzdata::ETC_GMT_PLUS_2,
    raw_tzdata::CET,
    raw_tzdata::ETC_GMT_MINUS_6,
    raw_tzdata::EST_5_EDT,
    raw_tzdata::ETC_GMT_MINUS_5,
    raw_tzdata::AMERICA_CAYENNE,
    raw_tzdata::PACIFIC_RAROTONGA,
    raw_tzdata::ETC_GMT_PLUS_6,
    raw_tzdata::ETC_GMT_PLUS_5,
    raw_tzdata::AMERICA_BELEM,
    raw_tzdata::ETC_GMT_MINUS_9,
    raw_tzdata::AMERICA_SAO_PAULO,
    raw_tzdata::ETC_GMT_MINUS_4,
    raw_tzdata::AUSTRALIA_CURRIE,
    raw_tzdata::ETC_GMT_MINUS_3,
    raw_tzdata::ETC_GMT_PLUS_9,
    raw_tzdata::ETC_GMT_PLUS_4,
    raw_tzdata::ASIA_CHOIBALSAN,
    raw_tzdata::ETC_GMT_PLUS_3,
    raw_tzdata::AFRICA_TUNIS,
    raw_tzdata::AMERICA_GOOSE_BAY,
    raw_tzdata::AMERICA_DANMARKSHAVN,
    raw_tzdata::AMERICA_EL_SALVADOR,
    raw_tzdata::AUSTRALIA_DARWIN,
    raw_tzdata::AUSTRALIA_ADELAIDE,
    raw_tzdata::AMERICA_SWIFT_CURRENT,
    raw_tzdata::PORTUGAL,
    raw_tzdata::HST,
    raw_tzdata::PST_8_PDT,
    raw_tzdata::AUSTRALIA_BRISBANE,
    raw_tzdata::MST,
    raw_tzdata::ASIA_PONTIANAK,
    raw_tzdata::AMERICA_GLACE_BAY,
    raw_tzdata::CHILE_EASTER_ISLAND,
    raw_tzdata::ASIA_SAMARKAND,
    raw_tzdata::CET,
    raw_tzdata::ASIA_DACCA,
    raw_tzdata::AMERICA_MONTERREY,
    raw_tzdata::AMERICA_MAZATLAN,
    raw_tzdata::AMERICA_ADAK,
    raw_tzdata::ROC,
    raw_tzdata::ASIA_GAZA,
    raw_tzdata::ASIA_VIENTIANE,
    raw_tzdata::AMERICA_ATIKOKAN,
    raw_tzdata::AFRICA_KAMPALA,
    raw_tzdata::ASIA_ATYRAU,
    raw_tzdata::ASIA_MAGADAN,
    raw_tzdata::AMERICA_MONTREAL,
    raw_tzdata::AMERICA_RANKIN_INLET,
    raw_tzdata::AMERICA_LOUISVILLE,
    raw_tzdata::AMERICA_KENTUCKY_MONTICELLO,
    raw_tzdata::ROK,
    raw_tzdata::AMERICA_DETROIT,
    raw_tzdata::CHILE_EASTER_ISLAND,
    raw_tzdata::CST_6_CDT,
    raw_tzdata::PACIFIC_NIUE,
    raw_tzdata::AFRICA_OUAGADOUGOU,
    raw_tzdata::ASIA_BAKU,
    raw_tzdata::AMERICA_ST_THOMAS,
    raw_tzdata::ATLANTIC_FAEROE,
    raw_tzdata::ASIA_DUSHANBE,
    raw_tzdata::ARCTIC_LONGYEARBYEN,
    raw_tzdata::ATLANTIC_FAEROE,
    raw_tzdata::ATLANTIC_BERMUDA,
    raw_tzdata::PRC,
    raw_tzdata::ATLANTIC_MADEIRA,
    raw_tzdata::AMERICA_CAMPO_GRANDE,
    raw_tzdata::AMERICA_ADAK,
    raw_tzdata::ASIA_ASHGABAT,
    raw_tzdata::AMERICA_NORTH_DAKOTA_BEULAH,
    raw_tzdata::AFRICA_KIGALI,
    raw_tzdata::AUSTRALIA_EUCLA,
    raw_tzdata::AMERICA_ST_JOHNS,
    raw_tzdata::AMERICA_KNOX_IN,
    raw_tzdata::AMERICA_GRAND_TURK,
    raw_tzdata::ASIA_PYONGYANG,
    raw_tzdata::AMERICA_BAHIA,
    raw_tzdata::AMERICA_CIUDAD_JUAREZ,
    raw_tzdata::AFRICA_NIAMEY,
    raw_tzdata::AFRICA_NOUAKCHOTT,
    raw_tzdata::AMERICA_JUJUY,
    raw_tzdata::AMERICA_WHITEHORSE,
    raw_tzdata::AUSTRALIA_ACT,
    raw_tzdata::AFRICA_BUJUMBURA,
    raw_tzdata::AUSTRALIA_BRISBANE,
    raw_tzdata::PACIFIC_TONGATAPU,
    raw_tzdata::AUSTRALIA_ACT,
    raw_tzdata::ASIA_YEKATERINBURG,
    raw_tzdata::AMERICA_INDIANA_TELL_CITY,
    raw_tzdata::HONGKONG,
    raw_tzdata::AMERICA_SITKA,
    raw_tzdata::EUROPE_MALTA,
    raw_tzdata::AMERICA_INDIANA_VEVAY,
    raw_tzdata::EUROPE_SAMARA,
    raw_tzdata::AMERICA_GUAYAQUIL,
    raw_tzdata::ASIA_KHANDYGA,
    raw_tzdata::EUROPE_SOFIA,
    raw_tzdata::AMERICA_NORONHA,
    raw_tzdata::AFRICA_GABORONE,
    raw_tzdata::AFRICA_DAR_ES_SALAAM,
    raw_tzdata::AUSTRALIA_PERTH,
    raw_tzdata::EUROPE_HELSINKI,
    raw_tzdata::ASIA_TASHKENT,
    raw_tzdata::MST_7_MDT,
    raw_tzdata::EUROPE_KIEV,
    raw_tzdata::ASIA_KATHMANDU,
    raw_tzdata::EUROPE_COPENHAGEN,
    raw_tzdata::JAPAN,
    raw_tzdata::ASIA_ALMATY,
    raw_tzdata::ASIA_TBILISI,
    raw_tzdata::AMERICA_CAYMAN,
    raw_tzdata::EUROPE_MADRID,
    raw_tzdata::AMERICA_KRALENDIJK,
    raw_tzdata::UCT,
    raw_tzdata::EUROPE_ANDORRA,
    raw_tzdata::AMERICA_MANAUS,
    raw_tzdata::EUROPE_ZAGREB,
    raw_tzdata::PACIFIC_NAURU,
    raw_tzdata::TURKEY,
    raw_tzdata::ARCTIC_LONGYEARBYEN,
    raw_tzdata::AFRICA_LUBUMBASHI,
    raw_tzdata::PACIFIC_GUADALCANAL,
    raw_tzdata::AUSTRALIA_MELBOURNE,
    raw_tzdata::ATLANTIC_SOUTH_GEORGIA,
    raw_tzdata::AMERICA_VANCOUVER,
    raw_tzdata::EUROPE_BELGRADE,
    raw_tzdata::ASIA_THIMBU,
    raw_tzdata::AFRICA_JUBA,
    raw_tzdata::AMERICA_ANCHORAGE,
    raw_tzdata::AFRICA_EL_AAIUN,
    raw_tzdata::ANTARCTICA_ROTHERA,
    raw_tzdata::AMERICA_REGINA,
    raw_tzdata::ASIA_TOMSK,
    raw_tzdata::AMERICA_BAHIA_BANDERAS,
    raw_tzdata::AMERICA_CHIHUAHUA,
    raw_tzdata::ASIA_KABUL,
    raw_tzdata::ASIA_KASHGAR,
    raw_tzdata::AFRICA_JOHANNESBURG,
    raw_tzdata::ROK,
    raw_tzdata::AMERICA_CAMBRIDGE_BAY,
    raw_tzdata::ASIA_NOVOSIBIRSK,
    raw_tzdata::AUSTRALIA_CURRIE,
    raw_tzdata::AMERICA_ST_LUCIA,
    raw_tzdata::ASIA_ORAL,
    raw_tzdata::EUROPE_ROME,
    raw_tzdata::PACIFIC_GAMBIER,
    raw_tzdata::INDIAN_CHAGOS,
    raw_tzdata::EUROPE_RIGA,
    raw_tzdata::AFRICA_BAMAKO,
    raw_tzdata::AMERICA_ST_VINCENT,
    raw_tzdata::PACIFIC_PAGO_PAGO,
    raw_tzdata::AUSTRALIA_LHI,
    raw_tzdata::AUSTRALIA_LINDEMAN,
    raw_tzdata::ASIA_NICOSIA,
    raw_tzdata::LIBYA,
    raw_tzdata::PACIFIC_PAGO_PAGO,
    raw_tzdata::AUSTRALIA_ACT,
    raw_tzdata::EGYPT,
    raw_tzdata::EUROPE_TIRANE,
    raw_tzdata::AUSTRALIA_MELBOURNE,
    raw_tzdata::PACIFIC_NOUMEA,
    raw_tzdata::PACIFIC_MARQUESAS,
    raw_tzdata::EET,
    raw_tzdata::EUROPE_HELSINKI,
    raw_tzdata::ASIA_JAKARTA,
    raw_tzdata::UCT,
    raw_tzdata::EUROPE_ASTRAKHAN,
    raw_tzdata::AFRICA_BANJUL,
    raw_tzdata::EUROPE_MONACO,
    raw_tzdata::AMERICA_HERMOSILLO,
    raw_tzdata::AMERICA_MONTREAL,
    raw_tzdata::NZ_CHAT,
    raw_tzdata::AUSTRALIA_BROKEN_HILL,
    raw_tzdata::AMERICA_ST_JOHNS,
    raw_tzdata::EUROPE_BRATISLAVA,
    raw_tzdata::AMERICA_PORTO_ACRE,
    raw_tzdata::INDIAN_CHRISTMAS,
    raw_tzdata::PACIFIC_SAIPAN,
    raw_tzdata::FACTORY,
    raw_tzdata::ANTARCTICA_SYOWA,
    raw_tzdata::PACIFIC_ENDERBURY,
    raw_tzdata::EUROPE_BRATISLAVA,
    raw_tzdata::ASIA_KASHGAR,
    raw_tzdata::ATLANTIC_CAPE_VERDE,
    raw_tzdata::NZ,
    raw_tzdata::ANTARCTICA_DAVIS,
    raw_tzdata::EUROPE_VIENNA,
    raw_tzdata::PACIFIC_PITCAIRN,
    raw_tzdata::AUSTRALIA_BROKEN_HILL,
    raw_tzdata::ASIA_BARNAUL,
    raw_tzdata::AMERICA_DAWSON_CREEK,
    raw_tzdata::EUROPE_BELGRADE,
    raw_tzdata::ANTARCTICA_DUMONT_D_URVILLE,
    raw_tzdata::CUBA,
    raw_tzdata::AMERICA_KRALENDIJK,
    raw_tzdata::EUROPE_PARIS,
    raw_tzdata::EUROPE_SARAJEVO,
    raw_tzdata::AMERICA_EDMONTON,
    raw_tzdata::PACIFIC_FUNAFUTI,
    raw_tzdata::GB,
    raw_tzdata::PRC,
    raw_tzdata::PACIFIC_GALAPAGOS,
    raw_tzdata::AMERICA_JUJUY,
    raw_tzdata::PORTUGAL,
    raw_tzdata::AMERICA_SANTIAGO,
    raw_tzdata::AFRICA_BRAZZAVILLE,
    raw_tzdata::MST_7_MDT,
    raw_tzdata::EUROPE_VOLGOGRAD,
    raw_tzdata::AMERICA_HALIFAX,
    raw_tzdata::EUROPE_BUSINGEN,
    raw_tzdata::PACIFIC_GUAM,
    raw_tzdata::EUROPE_SARATOV,
    raw_tzdata::ATLANTIC_STANLEY,
    raw_tzdata::ANTARCTICA_MAWSON,
    raw_tzdata::ANTARCTICA_MC_MURDO,
    raw_tzdata::PACIFIC_BOUGAINVILLE,
    raw_tzdata::ATLANTIC_CANARY,
    raw_tzdata::AFRICA_LUSAKA,
    raw_tzdata::ASIA_KARACHI,
    raw_tzdata::ANTARCTICA_MACQUARIE,
    raw_tzdata::EUROPE_KALININGRAD,
    raw_tzdata::ASIA_KUCHING,
    raw_tzdata::AMERICA_INUVIK,
    raw_tzdata::HST,
    raw_tzdata::EUROPE_VILNIUS,
    raw_tzdata::AMERICA_MEXICO_CITY,
    raw_tzdata::PST_8_PDT,
    raw_tzdata::ANTARCTICA_PALMER,
    raw_tzdata::AMERICA_ENSENADA,
    raw_tzdata::ATLANTIC_ST_HELENA,
    raw_tzdata::W_SU,
    raw_tzdata::AFRICA_CONAKRY,
    raw_tzdata::EUROPE_ROME,
    raw_tzdata::ANTARCTICA_TROLL,
    raw_tzdata::POLAND,
    raw_tzdata::ASIA_JAYAPURA,
    raw_tzdata::ASIA_CALCUTTA,
    raw_tzdata::EUROPE_OSLO,
    raw_tzdata::PACIFIC_MAJURO,
    raw_tzdata::UCT,
    raw_tzdata::AMERICA_YAKUTAT,
    raw_tzdata::UCT,
    raw_tzdata::PACIFIC_ENDERBURY,
    raw_tzdata::ASIA_VLADIVOSTOK,
    raw_tzdata::AFRICA_KINSHASA,
    raw_tzdata::ATLANTIC_AZORES,
    raw_tzdata::AMERICA_SAO_PAULO,
    raw_tzdata::EUROPE_TALLINN,
    raw_tzdata::EIRE,
    raw_tzdata::PACIFIC_KIRITIMATI,
    raw_tzdata::ASIA_KATHMANDU,
    raw_tzdata::PACIFIC_GUADALCANAL,
    raw_tzdata::ARCTIC_LONGYEARBYEN,
    raw_tzdata::PACIFIC_TAHITI,
    raw_tzdata::AMERICA_COYHAIQUE,
    raw_tzdata::AMERICA_MAZATLAN,
    raw_tzdata::PACIFIC_PALAU,
    raw_tzdata::AUSTRALIA_CURRIE,
    raw_tzdata::PACIFIC_PORT_MORESBY,
    raw_tzdata::AUSTRALIA_DARWIN,
    raw_tzdata::EUROPE_AMSTERDAM,
    raw_tzdata::ASIA_SAKHALIN,
    raw_tzdata::PACIFIC_FIJI,
    raw_tzdata::ASIA_HO_CHI_MINH,
    raw_tzdata::AMERICA_ADAK,
    raw_tzdata::UCT,
    raw_tzdata::CET,
    raw_tzdata::EST_5_EDT,
    raw_tzdata::AFRICA_KHARTOUM,
    raw_tzdata::PACIFIC_KOSRAE,
    raw_tzdata::EUROPE_CHISINAU,
    raw_tzdata::EUROPE_BUDAPEST,
    raw_tzdata::AMERICA_ST_KITTS,
    raw_tzdata::ASIA_NOVOKUZNETSK,
    raw_tzdata::HST,
    raw_tzdata::NZ,
    raw_tzdata::ASIA_KUALA_LUMPUR,
    raw_tzdata::AUSTRALIA_ADELAIDE,
    raw_tzdata::GB,
    raw_tzdata::ASIA_PHNOM_PENH,
    raw_tzdata::PACIFIC_WAKE,
    raw_tzdata::PACIFIC_MIDWAY,
    raw_tzdata::ANTARCTICA_CASEY,
    raw_tzdata::ISRAEL,
    raw_tzdata::ASIA_MAKASSAR,
    raw_tzdata::KWAJALEIN,
    raw_tzdata::AMERICA_GODTHAB,
    raw_tzdata::AMERICA_LA_PAZ,
    raw_tzdata::EUROPE_LUXEMBOURG,
    raw_tzdata::ISRAEL,
    raw_tzdata::TURKEY,
    raw_tzdata::KWAJALEIN,
    raw_tzdata::EUROPE_ROME,
    raw_tzdata::ASIA_KRASNOYARSK,
    raw_tzdata::EUROPE_GIBRALTAR,
    raw_tzdata::PACIFIC_WALLIS,
    raw_tzdata::HST,
    raw_tzdata::PACIFIC_PORT_MORESBY,
    raw_tzdata::ASIA_BANGKOK,
    raw_tzdata::AUSTRALIA_ACT,
    raw_tzdata::AFRICA_WINDHOEK,
    raw_tzdata::PACIFIC_POHNPEI,
    raw_tzdata::ASIA_OMSK,
    raw_tzdata::ASIA_BAGHDAD,
    raw_tzdata::ASIA_THIMBU,
    raw_tzdata::AUSTRALIA_PERTH,
    raw_tzdata::EUROPE_SIMFEROPOL,
    raw_tzdata::EUROPE_KIEV,
    raw_tzdata::AUSTRALIA_LHI,
    raw_tzdata::EUROPE_LJUBLJANA,
    raw_tzdata::GMT,
    raw_tzdata::EUROPE_ISLE_OF_MAN,
    raw_tzdata::AMERICA_KRALENDIJK,
    raw_tzdata::PACIFIC_CHUUK,
    raw_tzdata::PACIFIC_FAKAOFO,
    raw_tzdata::PACIFIC_NORFOLK,
    raw_tzdata::TURKEY,
    raw_tzdata::EUROPE_KIEV,
    raw_tzdata::EUROPE_KIROV,
    raw_tzdata::EUROPE_CHISINAU,
    raw_tzdata::PORTUGAL,
    raw_tzdata::ATLANTIC_REYKJAVIK,
    raw_tzdata::EUROPE_BUCHAREST,
    raw_tzdata::ASIA_SREDNEKOLYMSK,
    raw_tzdata::EUROPE_BUSINGEN,
    raw_tzdata::EUROPE_STOCKHOLM,
    raw_tzdata::ANTARCTICA_VOSTOK,
    raw_tzdata::ASIA_RIYADH,
    raw_tzdata::EUROPE_VADUZ,
    raw_tzdata::EUROPE_JERSEY,
    raw_tzdata::EUROPE_SKOPJE,
    raw_tzdata::PACIFIC_PORT_MORESBY,
    raw_tzdata::AMERICA_MEXICO_CITY,
    raw_tzdata::GMT,
    raw_tzdata::EUROPE_GUERNSEY,
    raw_tzdata::HONGKONG,
    raw_tzdata::EUROPE_ULYANOVSK,
    raw_tzdata::ASIA_IRKUTSK,
    raw_tzdata::EUROPE_KIEV,
    raw_tzdata::ASIA_BISHKEK,
    raw_tzdata::EUROPE_MINSK,
    raw_tzdata::ASIA_YAKUTSK,
];

const ASSO_VALUES: [u16; 257] = [
    2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522,
    2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522,
    2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 4, 11, 3, 5, 481, 3, 7, 19,
    33, 32, 25, 24, 18, 9, 31, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 5, 39, 183, 23, 7, 516,
    75, 758, 16, 284, 715, 340, 265, 8, 8, 310, 9, 14, 136, 2, 148, 358, 231, 73, 444, 443, 15,
    2522, 2522, 2522, 499, 2, 5, 39, 183, 23, 7, 516, 75, 758, 16, 284, 715, 340, 265, 8, 8, 310,
    9, 14, 136, 2, 148, 358, 231, 73, 444, 443, 15, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522,
    2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522,
    2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522,
    2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522,
    2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522,
    2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522,
    2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522,
    2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522,
    2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522, 2522,
];

pub(crate) const fn find_key(s: &[u8]) -> Option<Index> {
    let len = s.len();
    if !matches!(len, 2..=32) {
        return None;
    }

    let mut key: usize = len;
    key = key.wrapping_add(ASSO_VALUES[s[len - 1] as usize] as usize);
    if len >= 1 {
        key = key.wrapping_add(ASSO_VALUES[s[0] as usize] as usize);
    }
    if len >= 2 {
        key = key.wrapping_add(ASSO_VALUES[(s[1] as usize).wrapping_add(1)] as usize);
    }
    if len >= 4 {
        key = key.wrapping_add(ASSO_VALUES[s[3] as usize] as usize);
    }
    if len >= 6 {
        key = key.wrapping_add(ASSO_VALUES[(s[5] as usize).wrapping_add(1)] as usize);
    }
    if len >= 8 {
        key = key.wrapping_add(ASSO_VALUES[(s[7] as usize).wrapping_add(1)] as usize);
    }
    if len >= 9 {
        key = key.wrapping_add(ASSO_VALUES[s[8] as usize] as usize);
    }
    if len >= 11 {
        key = key.wrapping_add(ASSO_VALUES[s[10] as usize] as usize);
    }
    if len >= 12 {
        key = key.wrapping_add(ASSO_VALUES[s[11] as usize] as usize);
    }
    if len > 18 {
        key = key.wrapping_add(ASSO_VALUES[s[18] as usize] as usize);
    }

    if key > 2521 {
        return None;
    }
    let key = match WORDLIST[key] {
        Some(key) => key,
        None => return None,
    };
    if !eq_ignore_ascii_case(s, NAMES[key as u16 as usize]) {
        return None;
    }

    Some(key)
}
