//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use core::ptr::NonNull;
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

extern_class!(
    /// [Apple's documentation](https://developer.apple.com/documentation/cloudkit/ckuseridentity?language=objc)
    #[unsafe(super(NSObject))]
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct CKUserIdentity;
);

extern_conformance!(
    unsafe impl NSCoding for CKUserIdentity {}
);

extern_conformance!(
    unsafe impl NSCopying for CKUserIdentity {}
);

unsafe impl CopyingHelper for CKUserIdentity {
    type Result = Self;
}

extern_conformance!(
    unsafe impl NSObjectProtocol for CKUserIdentity {}
);

extern_conformance!(
    unsafe impl NSSecureCoding for CKUserIdentity {}
);

impl CKUserIdentity {
    extern_methods!(
        /// Use
        /// `CKDiscoverUserIdentitiesOperation`or
        /// `CKFetchShareParticipantsOperation`to create a
        /// `CKUserIdentity`
        #[unsafe(method(init))]
        #[unsafe(method_family = init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[unsafe(method(new))]
        #[unsafe(method_family = new)]
        pub unsafe fn new() -> Retained<Self>;

        #[cfg(feature = "CKRecordID")]
        #[unsafe(method(userRecordID))]
        #[unsafe(method_family = none)]
        pub unsafe fn userRecordID(&self) -> Option<Retained<CKRecordID>>;

        #[cfg(feature = "CKUserIdentityLookupInfo")]
        /// This is the
        /// `lookupInfo`you passed in to
        /// `CKDiscoverUserIdentitiesOperation`or
        /// `CKFetchShareParticipantsOperation`
        #[unsafe(method(lookupInfo))]
        #[unsafe(method_family = none)]
        pub unsafe fn lookupInfo(&self) -> Option<Retained<CKUserIdentityLookupInfo>>;

        #[unsafe(method(nameComponents))]
        #[unsafe(method_family = none)]
        pub unsafe fn nameComponents(&self) -> Option<Retained<NSPersonNameComponents>>;

        #[unsafe(method(hasiCloudAccount))]
        #[unsafe(method_family = none)]
        pub unsafe fn hasiCloudAccount(&self) -> bool;

        #[deprecated = "No longer supported. Please see Sharing CloudKit Data with Other iCloud Users."]
        #[unsafe(method(contactIdentifiers))]
        #[unsafe(method_family = none)]
        pub unsafe fn contactIdentifiers(&self) -> Retained<NSArray<NSString>>;
    );
}
